# Standard library imports
from typing import Any, Dict, List, Optional

# Third party imports
import requests

# Local imports
from .enums import ModelType, OrderResult
from .models import (
    AppProfile,
    BugProfile,
    ClubProfile,
    GameManagerResponse,
    GameProfile,
    IdeaProfile,
    Member,
    MemberProfile,
    ModeratorResponse,
    ModProfile,
    ResultResponse,
    StudioProfile,
    OnlineResponse,
    DiscussionResponse,
)

class PyBanana:
    """GameBanana API client."""
    
    def __init__(self):
        self.base_url = "https://gamebanana.com/apiv11"
        self.headers = {"Accept": "application/json", "User-Agent": "Python/PyBanana"}

    def _get_data(self, endpoint: str, params: Optional[Dict[str, Any]] = None) -> Any:
        """Make a GET request to the GameBanana API."""
        url = f"{self.base_url}{endpoint}"
        if params is None:
            params = {}
        params["format"] = "json"  # Ensure JSON response
        response = requests.get(url, params=params, headers=self.headers)
        response.raise_for_status()
        return response.json()
    
    def _get_auth_cookies(self, username: str, password: str):
        """Authenticate with GameBanana and get session cookie."""   
        url = "https://gamebanana.com/apiv11/Member/Authenticate"
        json_data = { "_sUsername": username, "_sPassword": password }
        response = requests.post(url, json=json_data, headers=self.headers)
        response.raise_for_status()
        return response.cookies
          
    def get_member(self, user_id: int) -> Optional[Member]:
        """Get information about a specific user by ID."""
        try:
            data = self._get_data(f"/Member/{user_id}")
            return Member.from_dict(data)
        except Exception as e:
            return None

    def get_member_profile(self, user_id: int) -> Optional[MemberProfile]:
        """Get detailed profile information about a specific user."""
        try:
            data = self._get_data(f"/Member/{user_id}/ProfilePage")
            return MemberProfile.from_dict(data)
        except Exception as e:
            return None

    def get_game_profile(self, game_id: int) -> Optional[GameProfile]:
        """Get information about a specific game by ID."""
        try:
            data = self._get_data(f"/Game/{game_id}/ProfilePage")
            return GameProfile.from_dict(data)
        except Exception as e:
            return None

    def get_app_profile(self, app_id: int) -> Optional[AppProfile]:
        """Get information about a specific app by ID."""
        try:
            data = self._get_data(f"/App/{app_id}/ProfilePage")
            return AppProfile.from_dict(data)
        except Exception as e:
            return None

    def get_bug_profile(self, bug_id: int) -> Optional[BugProfile]:
        """Get information about a specific bug report by ID."""
        try:
            data = self._get_data(f"/Bug/{bug_id}/ProfilePage")
            return BugProfile.from_dict(data)
        except Exception as e:
            return None

    def get_idea_profile(self, idea_id: int) -> Optional[IdeaProfile]:
        """Get information about a specific idea by ID."""
        try:
            data = self._get_data(f"/Idea/{idea_id}/ProfilePage")
            return IdeaProfile.from_dict(data)
        except Exception as e:
            return None
    
    def get_mod_profile(self, submission_id: int) -> Optional[ModProfile]:
        """Get detailed information about a specific submission."""
        try:
            data = self._get_data(f"/Mod/{submission_id}/ProfilePage")
            return ModProfile.from_dict(data)
        except Exception as e:
            return None
    
    def get_studio_profile(self, studio_id: int) -> Optional[StudioProfile]:
        """Get information about a specific studio by ID."""
        try:
            data = self._get_data(f"/Studio/{studio_id}/ProfilePage")
            return StudioProfile.from_dict(data)
        except Exception as e:
            return None
    
    def get_club_profile(self, club_id: int) -> Optional[ClubProfile]:
        """Get information about a specific club by ID."""
        try:
            data = self._get_data(f"/Club/{club_id}/ProfilePage")
            return ClubProfile.from_dict(data)
        except Exception as e:
            return None

    def get_moderators(self) -> Optional[ModeratorResponse]:
        """Get a list of moderators."""
        try:
            data = self._get_data("/Member/Moderators")
            return ModeratorResponse.from_dict(data)
        except Exception as e:
            return None

    def get_managers(self, page: int = 1, per_page: int = 15) -> Optional[GameManagerResponse]:
        """Get a list of game managers with pagination."""
        try:
            params = { "_nPage": page, "_nPerpage": per_page }
            data = self._get_data("/Member/GameManagers", params=params)
            return GameManagerResponse.from_dict(data)
        except Exception as e:
            return None

    def get_online_members(self, page: int = 1, per_page: int = 15) -> Optional[OnlineResponse]:
        """Get a list of currently online members."""
        try:
            params = { "_nPage": page, "_nPerpage": per_page } 
            data = self._get_data("/Member/Online", params=params)
            return OnlineResponse.from_dict(data)
        except Exception as e:
            return None
        
    def get_discussions(self, page: int = 1, per_page: int = 15) -> Optional[DiscussionResponse]:
        """Get a list of recent discussions."""
        try:
            params = { "_nPage": page, "_nPerpage": per_page }
            data = self._get_data("/Util/Generic/Discussions", params=params)
            return DiscussionResponse.from_dict(data)
        except Exception as e:
            return None

    def get_download_url(self, model_name: ModelType, item_id: int) -> Optional[str]:
        """Get the download URL for a specific file."""
        try:
            data = self._get_data(f"/{model_name}/{item_id}/ProfilePage")
            return data.get("_sDownloadUrl", "")
        except Exception as e:
            return None
     
    def search(
        self,
        query: str,
        model: ModelType,
        order: OrderResult = OrderResult.RELEVANCE,
        page: int = 1,
        per_page: int = 15,
        fields: Optional[str] = None,
    ) -> Optional[ResultResponse]:
        """Search for content across GameBanana."""
        try:
            params = {
                "_sSearchString": query,
                "_sModelName": model.value,
                "_sOrder": order.value,
                "_nPage": page,
                "_nPerpage": per_page,
                "_csvFields": fields if fields is not None else "name,description,article,attribs,studio,owner,credits"
            }
            
            data = self._get_data("/Util/Search/Results", params=params)
            return ResultResponse.from_dict(data)
        except Exception as e:
            return None



from .bio import Bio, BioEntry
from .category import ModCategory
from .field import ProfileField, ContactInfo, PcSpecs, SoftwareKit, GamingDevices
from .credits import Author, Credits, CreditGroup, AffiliatedStudio
from .embeddable import Embeddable
from .online import OnlineStatus, OnlineRecord
from .preview import PreviewMedia, PreviewMediaImage
from .profile import Profile
from .ratings import RatingsSummary, RatingBreakdownItem
from .stats import CoreStats
from .buddy import Buddy, SubjectShaper
from .managers import ManagerRecord
from .moderators import ModeratorRecord
from .discussion import DiscussionRecord, Post, Submission
from .file import File
from .license import LicenseChecklist
from .medals import Medals

__all__ = [
    "Bio",
    "BioEntry",
    "ModCategory",
    "ManagerRecord",
    "ModeratorRecord",
    "OnlineRecord",
    "DiscussionRecord",
    "Post",
    "Submission",
    "File",
    "LicenseChecklist",

    "Buddy",
    "SubjectShaper",

    "Author",
    "Credits",
    "CreditGroup",
    "AffiliatedStudio",

    "ContactInfo",
    "PcSpecs",
    "SoftwareKit",
    "GamingDevices",
    "Medals",

    "Embeddable",
    "OnlineStatus",

    "PreviewMedia",
    "PreviewMediaImage",

    "Profile",
    "ProfileField",
    "RatingsSummary",
    "RatingBreakdownItem",
    "CoreStats"
]






from dataclasses import dataclass
from typing import List, Optional
from datetime import datetime

@dataclass
class Medals:
    """A GameBanana achievement medal."""
    id: int = 0
    image_url: str = ""
    name: str = ""
    grade: str = ""
    grade_title: str = ""
    date_added: Optional[datetime] = None

    @classmethod
    def from_dict(cls, data: dict) -> "Medals":
        """Create a Medals instance from a dictionary."""
        if not isinstance(data, dict):
            return cls()

        try:
            timestamp = data.get("_tsDateAdded", 0)
            date_added = datetime.fromtimestamp(timestamp) if timestamp else None
        except (ValueError, OSError):
            date_added = None

        return cls(
            id=data.get("_idRow", 0) or 0,
            image_url=data.get("_sImage", "") or "",
            name=data.get("_sName", "") or "",
            grade=data.get("_sGrade", "") or "",
            grade_title=data.get("_sGradeTitle", "") or "",
            date_added=date_added
        )


"""Bio entry related shared components."""
from dataclasses import dataclass, field
from typing import Dict, List, Optional, Union, Any

@dataclass
class BioEntry:
    title: str = ""
    value: str = ""
    custom_title: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, str]) -> "BioEntry":
        if not isinstance(data, dict):
            return cls()
            
        return cls(
            title=data.get("_sTitle", "") or "",
            value=data.get("_sValue", "") or "",
            custom_title=data.get("_sCustomTitle", "") or ""
        )

@dataclass
class Bio:
    entries: List[BioEntry] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Union[Dict[str, Any], List[Dict[str, Any]]]) -> "Bio":
        if not isinstance(data, (dict, list)):
            return cls()
            
        if isinstance(data, list):
            return cls()
            
        # Handle direct bio entry format
        if all(key in data for key in ["_sTitle", "_sValue"]):
            return cls(entries=[BioEntry.from_dict(data)])
            
        # Handle wrapped bio entries format
        bio_entries = data.get("_aBio", []) or []
        if not isinstance(bio_entries, list):
            bio_entries = []
            
        entries = [BioEntry.from_dict(entry) for entry in bio_entries]
        return cls(entries=entries)


"""Result models for GameBanana."""
from dataclasses import dataclass, field
from typing import List, Optional, Dict, Any
from .common.preview import PreviewMedia
from .member import Member
from .common.file import File

@dataclass
class Result:
    """Result information for a search query."""
    id_row: Optional[int]
    model_name: Optional[str]
    singular_title: Optional[str] 
    icon_classes: Optional[str]
    name: Optional[str]
    profile_url: Optional[str]
    date_added: Optional[int]
    date_modified: Optional[int]
    has_files: Optional[bool]
    preview_media: Optional[PreviewMedia] = None
    submitter: Optional[Member] = None
    initial_visibility: Optional[str] = None
    has_content_ratings: bool = False
    is_owned_by_accessor: bool = False
    files: List[File] = field(default_factory=list)
    accepts_donations: bool = False
    is_trashed: bool = False
    is_withheld: bool = False

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'Result':
        """Create a Result instance from a dictionary."""
        # Handle both search results and download page formats
        files = [File.from_dict(f) for f in data.get("_aFiles", [])] if "_aFiles" in data else []
        
        return cls(
            id_row=data.get("_idRow"),
            model_name=data.get("_sModelName"),
            singular_title=data.get("_sSingularTitle"),
            icon_classes=data.get("_sIconClasses"),
            name=data.get("_sName"),
            profile_url=data.get("_sProfileUrl"),
            date_added=data.get("_tsDateAdded"),
            date_modified=data.get("_tsDateModified"), 
            has_files=data.get("_bHasFiles"),
            preview_media=PreviewMedia.from_dict(data["_aPreviewMedia"]) if "_aPreviewMedia" in data and data["_aPreviewMedia"] else None,
            submitter=Member.from_dict(data["_aSubmitter"]) if "_aSubmitter" in data and data["_aSubmitter"] else None,
            initial_visibility=data.get("_sInitialVisibility"),
            has_content_ratings=data.get("_bHasContentRatings", False),
            is_owned_by_accessor=data.get("_bIsOwnedByAccessor", False),
            files=files,
            accepts_donations=data.get("_bAcceptsDonations", False),
            is_trashed=data.get("_bIsTrashed", False),
            is_withheld=data.get("_bIsWithheld", False)
        )


from dataclasses import dataclass
from typing import List, Optional
from datetime import datetime
from dataclasses import dataclass, field
from ..member import Member

@dataclass
class SubjectShaper:
    """A GameBanana subject shaper configuration."""
    border_style: str = ""
    font: str = ""
    text_color: str = ""
    text_hover_color: str = ""
    border_color: str = ""
    border_hover_color: str = ""
    
    @classmethod
    def from_dict(cls, data: dict) -> "SubjectShaper":
        return cls(
            border_style=data.get("_sBorderStyle", ""),
            font=data.get("_sFont", ""),
            text_color=data.get("_sTextColor", ""),
            text_hover_color=data.get("_sTextHoverColor", ""),
            border_color=data.get("_sBorderColor", ""),
            border_hover_color=data.get("_sBorderHoverColor", "")
        )

@dataclass
class Buddy(Member):
    """A GameBanana buddy relationship."""
    clearance_levels: List[str] = field(default_factory=list)
    hd_avatar_url: str = ""
    upic_url: str = ""
    subject_shaper: Optional[SubjectShaper] = None
    subject_shaper_css_code: str = ""
    date_added: Optional[datetime] = None

    @classmethod
    def from_dict(cls, data: dict) -> "Buddy":
        buddy_data = data.get("_aBuddy", {})
        member_data = super().from_dict(buddy_data).__dict__
        timestamp = data.get("_tsDateAdded", 0)
        try:
            date_added = datetime.fromtimestamp(timestamp) if timestamp > 0 else None
        except (OSError, ValueError):
            date_added = None
            
        return cls(
            id=member_data['id'],
            name=member_data['name'],
            is_online=member_data['is_online'],
            has_ripe=member_data['has_ripe'],
            profile_url=member_data['profile_url'],
            avatar_url=member_data['avatar_url'],
            clearance_levels=buddy_data.get("_aClearanceLevels", []),
            hd_avatar_url=buddy_data.get("_sHdAvatarUrl", ""),
            upic_url=buddy_data.get("_sUpicUrl", ""),
            subject_shaper=SubjectShaper.from_dict(buddy_data.get("_aSubjectShaper", {})),
            subject_shaper_css_code=buddy_data.get("_sSubjectShaperCssCode", ""),
            date_added=date_added
        )



"""Preview media related shared components."""
from dataclasses import dataclass, field
from typing import Dict, Any, List, Optional

@dataclass
class PreviewMediaImage:
    """An image in preview media."""
    url: str = ""
    width: int = 0
    height: int = 0
    caption: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "PreviewMediaImage":
        if not isinstance(data, dict):
            return cls()
            
        return cls(
            url=data.get("_sUrl", "") or "",
            width=data.get("_nWidth", 0) or 0,
            height=data.get("_nHeight", 0) or 0,
            caption=data.get("_sCaption", "") or ""
        )

@dataclass
class PreviewMedia:
    """Preview media information."""
    images: List[PreviewMediaImage] = field(default_factory=list)
    metadata: Dict[str, Any] = field(default_factory=dict)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "PreviewMedia":
        if not isinstance(data, dict):
            return cls()

        images = []
        image_data = data.get("_aImages", []) or []
        if isinstance(image_data, list):
            images = [PreviewMediaImage.from_dict(img) for img in image_data]

        metadata = data.get("_aMetadata", {})
        if not isinstance(metadata, dict):
            metadata = {}

        return cls(
            images=images,
            metadata=metadata
        )


from dataclasses import dataclass
from typing import List, Optional

@dataclass
class ProfileField:
    """Base class for formatted profile field entries."""
    title: str
    value: str
    input_type: Optional[str] = None
    icon_classes: Optional[str] = None
    value_template: Optional[str] = None
    formatted_value: Optional[str] = None

    @classmethod
    def from_dict(cls, data: dict) -> "ProfileField":
        """Create a ProfileField instance from a dictionary."""
        return cls(
            title=data["_sTitle"],
            value=data["_sValue"],
            input_type=data.get("_sInputType", ""),
            icon_classes=data.get("_sIconClasses", ""),
            value_template=data.get("_sValueTemplate", ""),
            formatted_value=data.get("_sFormattedValue", "")
        )
    
@dataclass
class ContactInfo(ProfileField):
    """A GameBanana contact information entry."""
    pass

@dataclass
class PcSpecs(ProfileField):
    """A GameBanana PC specification entry."""
    pass

@dataclass
class SoftwareKit(ProfileField):
    """A GameBanana software kit entry."""
    pass

@dataclass
class GamingDevices(ProfileField):
    """A GameBanana gaming devices entry."""
    pass


"""Credit-related shared components."""
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional

@dataclass
class AffiliatedStudio:
    profile_url: str = ""
    name: str = ""
    flag_url: str = ""
    banner_url: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AffiliatedStudio":
        if not isinstance(data, dict):
            return cls()
            
        return cls(
            profile_url=data.get("_sProfileUrl", "") or "",
            name=data.get("_sName", "") or "",
            flag_url=data.get("_sFlagUrl", "") or "",
            banner_url=data.get("_sBannerUrl", "") or ""
        )

@dataclass
class Author:
    role: str = ""
    name: str = ""
    id: Optional[int] = None
    upic_url: str = ""
    profile_url: str = ""
    is_online: bool = False
    affiliated_studio: Optional[AffiliatedStudio] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Author":
        if not isinstance(data, dict):
            return cls()
            
        affiliated_studio = None
        studio_data = data.get("_aAffiliatedStudio")
        if isinstance(studio_data, dict):
            affiliated_studio = AffiliatedStudio.from_dict(studio_data)

        return cls(
            role=data.get("_sRole", "") or "",
            name=data.get("_sName", "") or "",
            id=data.get("_idRow") or 0,
            upic_url=data.get("_sUpicUrl", "") or "",
            profile_url=data.get("_sProfileUrl", "") or "",
            is_online=bool(data.get("_bIsOnline", False)),
            affiliated_studio=affiliated_studio
        )

@dataclass
class CreditGroup:
    group_name: str = ""
    authors: List[Author] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CreditGroup":
        if not isinstance(data, dict):
            return cls()

        authors = []
        author_data = data.get("_aAuthors", []) or []
        if isinstance(author_data, list):
            authors = [Author.from_dict(author) for author in author_data]

        return cls(
            group_name=data.get("_sGroupName", "") or "",
            authors=authors
        )

@dataclass
class Credits:
    groups: List[CreditGroup] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Credits":
        if not isinstance(data, dict):
            return cls()

        groups = []
        groups_data = data.get("_aGroups", []) or []
        if isinstance(groups_data, list):
            groups = [CreditGroup.from_dict(group) for group in groups_data]

        return cls(groups=groups)


"""Bug profile and related functionality."""
from dataclasses import dataclass, field
from typing import Dict, Any, Optional, List
from datetime import datetime

from ..member import Member
from ..common.profile import Profile
from ..common.preview import PreviewMedia
from ..common.embeddable import Embeddable
from ..common.file import File  # Added missing import

@dataclass
class BugProfile:
    """Bug profile information."""
    base: Profile  # Required field
    status: int = 0
    is_private: bool = False
    date_modified: Optional[datetime] = None
    date_added: Optional[datetime] = None
    preview_media: Optional[PreviewMedia] = None
    accessor_is_submitter: bool = False
    is_trashed: bool = False
    post_count: int = 0
    thanks_count: int = 0
    initial_visibility: str = ""
    has_files: bool = False
    subscriber_count: int = 0
    text: str = ""
    resolution: str = ""
    resolution_key: str = ""
    priority: str = ""
    priority_key: str = ""
    source_url: str = ""
    show_ripe_promo: bool = False
    embeddables: List[Embeddable] = field(default_factory=list)
    submitter: Optional[Member] = None
    attachments: List[File] = field(default_factory=list)
    accessor_subscription_row_id: int = 0
    accessor_is_subscribed: bool = False
    accessor_has_thanked: bool = False

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "BugProfile":
        base = Profile.from_dict(data)
        preview_media_data = data.get("_aPreviewMedia", {})
        if isinstance(preview_media_data, list):
            preview_media_data = {}

        return cls(
            base=base,
            status=int(data.get("_nStatus", 0)),
            is_private=data.get("_bIsPrivate", False),
            date_modified=datetime.fromtimestamp(data.get("_tsDateModified", 0)),
            date_added=datetime.fromtimestamp(data.get("_tsDateAdded", 0)),
            preview_media=PreviewMedia.from_dict(preview_media_data),
            accessor_is_submitter=data.get("_bAccessorIsSubmitter", False),
            is_trashed=data.get("_bIsTrashed", False),
            post_count=data.get("_nPostCount", 0) or 0,
            thanks_count=data.get("_nThanksCount", 0) or 0,
            initial_visibility=data.get("_sInitialVisibility", "") or "",
            has_files=data.get("_bHasFiles", False) or False,
            subscriber_count=data.get("_nSubscriberCount", 0) or 0,
            text=data.get("_sText", "") or "",
            resolution=data.get("_sResolution", "") or "",
            resolution_key=data.get("_sResolutionKey", "") or "",
            priority=data.get("_sPriority", "") or "",
            priority_key=data.get("_sPriorityKey", "") or "", 
            source_url=data.get("_sSourceUrl", "") or "",
            show_ripe_promo=data.get("_bShowRipePromo", False) or False,
            embeddables=[Embeddable.from_dict(embed) for embed in data.get("_aEmbeddables", [])],
            submitter=Member.from_dict(data.get("_aMember", {})) if data.get("_aMember") else None,
            attachments=[File.from_dict(file) for file in data.get("_aAttachments", [])],
            accessor_subscription_row_id=data.get("_idAccessorSubscriptionRow", 0) or 0,
            accessor_is_subscribed=data.get("_bAccessorIsSubscribed", False) or False,
            accessor_has_thanked=data.get("_bAccessorHasThanked", False) or False
        )


"""GameBanana API models."""
# Base models
from .member import Member, Moderator, Manager
from .common.buddy import Buddy, SubjectShaper
from .common.profile import Profile
from .common.preview import PreviewMedia, PreviewMediaImage
from .common.category import ModCategory

# Common components
from .common.credits import Author, CreditGroup, Credits, AffiliatedStudio
from .common.embeddable import Embeddable
from .common.ratings import RatingsSummary, RatingBreakdownItem
from .common.stats import CoreStats
from .common.bio import Bio, BioEntry
from .common.field import ProfileField, ContactInfo, PcSpecs, SoftwareKit, GamingDevices
from .common.file import File
from .common.license import LicenseChecklist
from .common.medals import Medals
from .common.online import OnlineStatus
from .common.managers import ManagerRecord
from .common.moderators import ModeratorRecord
from .common.discussion import DiscussionRecord, Post, Submission
from .common.responses import (
    ModeratorResponse,
    GameManagerResponse,
    ResultResponse,
    OnlineResponse,
    DiscussionResponse
)

# Profile models
from .profiles.game import GameProfile, GameSection
from .profiles.app import AppProfile, AppFeatures
from .profiles.bug import BugProfile
from .profiles.idea import IdeaProfile
from .profiles.member import MemberProfile
from .profiles.mod import ModProfile
from .profiles.studio import StudioProfile, OpenPosition
from .profiles.club import ClubProfile

__all__ = [
    # Base models
    'Member', 'Moderator', 'Manager',
    'Buddy', 'SubjectShaper',
    'Profile',
    'PreviewMedia', 'PreviewMediaImage',
    'ModCategory',
    
    # Common components
    'Author', 'CreditGroup', 'Credits', 'AffiliatedStudio',
    'Embeddable',
    'RatingsSummary', 'RatingBreakdownItem',
    'CoreStats',
    'Bio', 'BioEntry',
    'ProfileField', 'ContactInfo', 'PcSpecs', 'SoftwareKit', 'GamingDevices',
    'File',
    'LicenseChecklist',
    'Medals',
    'OnlineStatus',
    'ManagerRecord',
    'ModeratorRecord',
    'DiscussionRecord', 'Post', 'Submission',
    
    # Response models
    'ModeratorResponse',
    'GameManagerResponse',
    'ResultResponse',
    'OnlineResponse',
    'DiscussionResponse',
    
    # Profile models
    'GameProfile', 'GameSection',
    'AppProfile', 'AppFeatures',
    'BugProfile',
    'IdeaProfile',
    'MemberProfile',
    'ModProfile',
    'StudioProfile', 'OpenPosition',
    'ClubProfile',
]


import pytest
from datetime import datetime
from pybanana.models.member import Member
from pybanana.models.common.profile import Profile
from pybanana.models.common.preview import PreviewMedia
from pybanana.models.common.category import ModCategory 
from pybanana.models.profiles.game import GameSection
from pybanana.models.common.discussion import Submission, Post, DiscussionRecord

def test_submission():
    test_data = {
        '_idRow': 123,
        '_sProfileUrl': 'https://gamebanana.com/discussions/123',
        '_sName': 'Test Discussion',
        '_nStatus': 1,
        '_bIsPrivate': False,
        '_tsDateModified': 1234567890,
        '_tsDateAdded': 1234567880,
        '_sModelName': 'Thread',
        '_sSingularTitle': 'Discussion Thread',
        '_sIconClasses': 'icon discussion',
        '_tsDateUpdated': 1234567895,
        '_aSubmitter': {
            '_idRow': 456,
            '_sName': 'TestUser',
            '_bIsOnline': True,
            '_bHasRipe': False,
            '_sProfileUrl': 'https://gamebanana.com/members/456',
            '_sAvatarUrl': 'https://gamebanana.com/avatars/456.jpg'
        },
        '_aGame': {
            '_sModelName': 'Mod',
            '_sPluralTitle': 'Mods',
            '_nItemCount': 150,
            '_nCategoryCount': 5,
            '_sUrl': 'https://gamebanana.com/games/123/mods'
        },
        '_aRootCategory': {
            '_idRow': 42,
            '_sName': 'Gameplay Mods',
            '_nItemCount': 100,
            '_nCategoryCount': 5,
            '_sUrl': 'https://gamebanana.com/games/123/mods/gameplay'
        },
        '_sVersion': '1.0.0',
        '_bIsObsolete': False,
        '_bHasContentRatings': True,
        '_nLikeCount': 10,
        '_nPostCount': 5,
        '_bWasFeatured': True,
        '_nViewCount': 100,
        '_bIsOwnedByAccessor': True
    }

    submission = Submission.from_dict(test_data)

    # Test base Profile fields
    assert isinstance(submission.base, Profile)
    assert submission.base.id == 123
    assert submission.base.profile_url == 'https://gamebanana.com/discussions/123'
    assert submission.base.name == 'Test Discussion'
    assert submission.base.status == 1
    assert submission.base.is_private is False
    assert isinstance(submission.base.date_modified, datetime)
    assert isinstance(submission.base.date_added, datetime)

    # Test Submission specific fields
    assert submission.model_name == 'Thread'
    assert submission.singular_title == 'Discussion Thread'
    assert submission.icon_classes == 'icon discussion'
    assert submission.date_updated == 1234567895
    assert isinstance(submission.submitter, Member)
    assert submission.submitter.id == 456
    assert submission.submitter.name == 'TestUser'
    assert isinstance(submission.game, GameSection)
    assert submission.game.model_name == 'Mod'
    assert isinstance(submission.root_category, ModCategory)
    assert submission.root_category.name == 'Gameplay Mods'
    assert submission.version == '1.0.0'
    assert submission.is_obsolete is False
    assert submission.has_content_ratings is True
    assert submission.like_count == 10
    assert submission.post_count == 5
    assert submission.was_featured is True
    assert submission.view_count == 100
    assert submission.is_owned_by_accessor is True

def test_submission_empty():
    test_data = {}
    submission = Submission.from_dict(test_data)
    assert submission is None

def test_post():
    test_data = {
        '_idRow': 789,
        '_tsDateAdded': 1234567890,
        '_sText': 'Test post content',
        '_aPoster': {
            '_idRow': 456,
            '_sName': 'TestUser',
            '_bIsOnline': True,
            '_bHasRipe': False,
            '_sProfileUrl': 'https://gamebanana.com/members/456',
            '_sAvatarUrl': 'https://gamebanana.com/avatars/456.jpg'
        }
    }

    post = Post.from_dict(test_data)

    assert post.id == 789
    assert post.date_added == 1234567890
    assert post.text == 'Test post content'
    assert isinstance(post.poster, Member)
    assert post.poster.id == 456
    assert post.poster.name == 'TestUser'

def test_post_empty():
    test_data = {}
    post = Post.from_dict(test_data)
    assert post is None

def test_discussion_record():
    test_data = {
        '_aSubmission': {
            '_idRow': 123,
            '_sProfileUrl': 'https://gamebanana.com/discussions/123',
            '_sName': 'Test Discussion',
            '_sModelName': 'Thread'
        },
        '_aPost': {
            '_idRow': 789,
            '_tsDateAdded': 1234567890,
            '_sText': 'Test post content',
            '_aPoster': {
                '_idRow': 456,
                '_sName': 'TestUser'
            }
        }
    }

    record = DiscussionRecord.from_dict(test_data)

    assert isinstance(record.submission, Submission)
    assert record.submission.base.id == 123
    assert record.submission.model_name == 'Thread'
    assert isinstance(record.post, Post)
    assert record.post.id == 789
    assert record.post.text == 'Test post content'

def test_discussion_record_empty():
    test_data = {}
    record = DiscussionRecord.from_dict(test_data)
    assert isinstance(record, DiscussionRecord)
    assert record.submission is None
    assert record.post is None


import pytest
from pybanana.api import PyBanana
from pybanana.enums import ModelType, OrderResult

@pytest.fixture
def api():
    return PyBanana()

def test_get_member(api):
    member = api.get_member(1382)  # Using Tom's ID
    assert member.id == 1382
    assert member.name == "tom"
    assert member.avatar_url is not None

def test_get_member_profile(api):
    profile = api.get_member_profile(1382)
    assert profile is not None
    assert hasattr(profile, "bio_entries")
    assert isinstance(profile.bio_entries, list)
    assert hasattr(profile, "show_ripe_promo")

def test_search(api):
    response = api.search(query="sound", model=ModelType.MOD, order=OrderResult.RELEVANCE)
    assert response is not None
    assert hasattr(response, "records")

def test_get_moderators(api):
    response = api.get_moderators()
    assert hasattr(response, "records")

def test_get_game_managers(api):
    response = api.get_game_managers(page=1, per_page=15)
    assert hasattr(response, "records")

def test_get_game_profile(api):
    # Using TF2's ID as it's a well-established game on GB
    game_profile = api.get_game_profile(297)
    assert game_profile is not None
    assert hasattr(game_profile, "name")
    assert game_profile.preview_media is not None

def test_api_error(api):
    with pytest.raises(Exception):
        api.get_member(999999999)

def test_get_auth_cookie(api):
    # Test with invalid credentials
    invalid_cookie = api._get_auth_cookie("invalid_user", "invalid_pass")
    assert invalid_cookie is None

    # Note: We can't test valid credentials here as that would require real GameBanana credentials
    # In a real test environment, you would mock the requests.post call


from dataclasses import dataclass
from typing import Optional, Any

@dataclass
class Member:
    """A GameBanana member."""
    id: int = 0
    name: str = ""
    is_online: bool = False
    has_ripe: bool = False
    profile_url: str = ""
    avatar_url: str = ""
    
    @classmethod
    def from_dict(cls, data: Any) -> Optional["Member"]:
        if not isinstance(data, dict):
            return None
            
        return cls(
            id=data.get("_idRow", 0) or 0,
            name=data.get("_sName", "") or "",
            is_online=bool(data.get("_bIsOnline", False)),
            has_ripe=bool(data.get("_bHasRipe", False)),
            profile_url=data.get("_sProfileUrl", "") or "",
            avatar_url=data.get("_sAvatarUrl", "") or ""
        )

@dataclass
class Moderator(Member):
    """Inherits base member attributes from Member class"""
    pass

@dataclass
class Manager(Member):
    """Inherits base member attributes from Member class"""
    pass




"""Online status related shared components."""
from dataclasses import dataclass
from typing import Dict, Any, Optional
from datetime import datetime

from ..member import Member

@dataclass
class OnlineStatus:
    is_online: bool = False
    location: str = ""
    last_seen_time: Optional[datetime] = None
    session_creation_time: Optional[datetime] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "OnlineStatus":
        return cls(
            is_online=data.get("_bIsOnline", False) or False,
            location=data.get("_sLocation", "") or "",
            last_seen_time=datetime.fromtimestamp(data.get("_tsLastSeenTime", 0) or 0),
            session_creation_time=datetime.fromtimestamp(data.get("_tsSessionCreationTime", 0) or 0)
        )
    
@dataclass
class OnlineRecord:
    member: Member
    status: OnlineStatus
    title: Optional[str] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "OnlineRecord":
        member = Member.from_dict(data.get('_aMember', {}))
        if member is None:
            return None
        return cls(
            member=member,
            status=OnlineStatus.from_dict(data.get('_aOnlineStatus', {})),
            title=data.get('_sOnlineTitle', "") or ""
        )



"""App profile and related functionality."""
from dataclasses import dataclass, field
from typing import List, Dict, Optional, Any

from ..common.bio import Bio
from ..common.profile import Profile
from ..common.credits import Credits
from ..common.category import ModCategory
from ..common.stats import CoreStats

@dataclass
class AppFeatures:
    """App features."""
    profile_module_url: str = ""
    navigator_tab_url: str = ""
    main_url: str = ""
    settings_url: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AppFeatures":
        return cls(
            profile_module_url=data.get("_sProfileModuleUrl", "") or "",
            navigator_tab_url=data.get("_sNavigatorTabUrl", "") or "",
            main_url=data.get("_sMainUrl", "") or "",
            settings_url=data.get("_sSettingsUrl", "") or ""
        )

@dataclass
class AppProfile:
    """App profile."""
    base: Profile
    description: str = ""
    include_variable_name: str = ""
    version: str = ""
    state: str = ""
    type: str = ""
    user_count: int = 0
    is_safe: bool = False
    accepts_donations: bool = False
    accessor_has_app: bool = False
    features: Optional[AppFeatures] = None
    credits: Optional[Credits] = None
    sticker_url: str = ""
    categories: List[ModCategory] = field(default_factory=list)
    bio: Optional[Bio] = None
    stats: Optional[CoreStats] = None

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "AppProfile":
        return cls(
            base=Profile.from_dict(data),
            description=data.get("_sDescription", "") or "",
            include_variable_name=data.get("_sIncludeVariableName", "") or "",
            version=data.get("_sVersion", "") or "",
            state=data.get("_sState", "") or "",
            type=data.get("_sType", "") or "",
            user_count=data.get("_nUserCount", 0) or 0,
            is_safe=data.get("_sbIsSafe", "") == "true",
            accepts_donations=data.get("_bAcceptsDonations", False),
            accessor_has_app=data.get("_bAccessorHasApp", False),
            features=AppFeatures.from_dict({} if isinstance(data.get("_aFeatures", {}), list) else data.get("_aFeatures", {})),
            credits=Credits.from_dict({} if isinstance(data.get("_aCredits", {}), list) else data.get("_aCredits", {})),
            sticker_url=data.get("_sStickerUrl", "") or "",
            categories=[ModCategory.from_dict(c) for c in data.get("_aCategories", {})],
            bio=Bio.from_dict({} if isinstance(data.get("_aBio", {}), list) else data.get("_aBio", {})),
            stats=CoreStats.from_dict({} if isinstance(data.get("_aStats"), list) else data.get("_aStats", {}))
        )



"""File-related shared components."""
from dataclasses import dataclass
from typing import Dict, Any, Optional
from datetime import datetime

@dataclass
class File:
    """File information for a mod."""
    id: int = 0
    filename: str = ""
    filesize: int = 0
    description: str = ""
    date_added: Optional[datetime] = None
    download_count: int = 0
    analysis_state: str = ""
    analysis_result_code: str = ""
    analysis_result: str = ""
    contains_exe: bool = False
    download_url: str = ""
    md5_checksum: str = ""
    clam_av_result: str = ""
    avast_av_result: str = ""

    def __post_init__(self):
        if self.date_added is None:
            self.date_added = datetime.fromtimestamp(0)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "File":
        try:
            timestamp = data.get("_tsDateAdded", 0)
            date_added = datetime.fromtimestamp(timestamp) if timestamp else None
        except (ValueError, OSError):
            date_added = None

        return cls(
            id=data.get("_idRow", 0) or 0,
            filename=data.get("_sFile", "") or "",
            filesize=data.get("_nFilesize", 0) or 0,
            description=data.get("_sDescription", "") or "",
            date_added=date_added,
            download_count=data.get("_nDownloadCount", 0) or 0,
            analysis_state=data.get("_sAnalysisState", "") or "",
            analysis_result_code=data.get("_sAnalysisResultCode", "") or "",
            analysis_result=data.get("_sAnalysisResult", "") or "",
            contains_exe=data.get("_bContainsExe", False),
            download_url=data.get("_sDownloadUrl", "") or "",
            md5_checksum=data.get("_sMd5Checksum", "") or "",
            clam_av_result=data.get("_sClamAvResult", "") or "",
            avast_av_result=data.get("_sAvastAvResult", "") or ""
        )


import pytest
from typing import get_type_hints
from pybanana.api import PyBanana
from pybanana.enums import ModelType
from pybanana.models.profiles.mod import ModProfile
from pybanana.models.profiles.game import GameProfile
from pybanana.models.profiles.member import MemberProfile 
from pybanana.models.profiles.studio import StudioProfile
from pybanana.models.profiles.club import ClubProfile
from pybanana.models.profiles.app import AppProfile
from pybanana.models.profiles.bug import BugProfile
from pybanana.models.profiles.idea import IdeaProfile
from pybanana.models.result import Result
from pybanana.models.common.responses import ModeratorResponse, GameManagerResponse, ResultResponse

def check_field_coverage(api_data: dict | list, model_class, model_name: str):
    """Helper function to check and report missing fields"""
    if isinstance(api_data, list):
        # For list responses, check fields in the first item
        if not api_data:
            return set()
        api_fields = set(api_data[0].keys())
    else:
        api_fields = set(api_data.keys())

    # Get fields from all class levels
    model_fields = set()
    
    # Get fields from model class
    hints = get_type_hints(model_class)
    model_fields.update(hints.keys())
    
    # Print missing fields for debugging
    missing_fields = api_fields - {f.lstrip('_') for f in model_fields}
    if missing_fields:
        print(f"\nMissing fields in {model_name}:")
        for field in sorted(missing_fields):
            print(f"- {field}")
            
    return api_fields

def test_mod_field_coverage():
    """Test that ModProfile covers all fields returned by the API"""
    api = PyBanana()
    mod_id = 572595  # known valid mod ID
    raw_data = api._get_data(f"/Mod/{mod_id}/ProfilePage")
    _ = ModProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, ModProfile, "ModProfile")

def test_game_field_coverage():
    """Test that GameProfile covers all fields returned by the API"""
    api = PyBanana()
    game_id = 297  # Team Fortress 2
    raw_data = api._get_data(f"/Game/{game_id}/ProfilePage")
    _ = GameProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, GameProfile, "GameProfile")

def test_member_field_coverage():
    """Test that MemberProfile covers all fields returned by the API"""
    api = PyBanana()
    member_id = 1382  # Tom
    raw_data = api._get_data(f"/Member/{member_id}/ProfilePage")
    _ = MemberProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, MemberProfile, "MemberProfile")

def test_studio_field_coverage():
    """Test that StudioProfile covers all fields returned by the API"""
    api = PyBanana()
    studio_id = 38059
    raw_data = api._get_data(f"/Studio/{studio_id}/ProfilePage")
    _ = StudioProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, StudioProfile, "StudioProfile")

def test_club_field_coverage():
    """Test that ClubProfile covers all fields returned by the API"""
    api = PyBanana()
    club_id = 28
    raw_data = api._get_data(f"/Club/{club_id}/ProfilePage")
    _ = ClubProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, ClubProfile, "ClubProfile")

def test_app_field_coverage():
    """Test that AppProfile covers all fields returned by the API"""
    api = PyBanana()
    app_id = 794
    raw_data = api._get_data(f"/App/{app_id}/ProfilePage")
    _ = AppProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, AppProfile, "AppProfile")

def test_bug_field_coverage():
    """Test that BugProfile covers all fields returned by the API"""
    api = PyBanana()
    bug_id = 4886
    raw_data = api._get_data(f"/Bug/{bug_id}/ProfilePage")
    _ = BugProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, BugProfile, "BugProfile")

def test_idea_field_coverage():
    """Test that IdeaProfile covers all fields returned by the API"""
    api = PyBanana()
    idea_id = 7079
    raw_data = api._get_data(f"/Idea/{idea_id}/ProfilePage")
    _ = IdeaProfile.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, IdeaProfile, "IdeaProfile")

def test_result_field_coverage():
    """Test that Result covers all fields returned by the API"""
    api = PyBanana()
    mod_id = 572595  # Use same valid mod ID
    raw_data = api._get_data(f"/Mod/{mod_id}/DownloadPage")
    _ = Result.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, Result, "Result")

def test_moderator_response_field_coverage():
    """Test that ModeratorResponse covers all fields returned by the API"""
    api = PyBanana()
    raw_data = api._get_data(f"/Member/Moderators")  # Changed to ModeratorList endpoint
    _ = ModeratorResponse.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, ModeratorResponse, "ModeratorResponse")

def test_game_manager_response_field_coverage():
    """Test that GameManagerResponse covers all fields returned by the API"""
    api = PyBanana()
    game_id = 297  # Team Fortress 2
    raw_data = api._get_data(f"/Game/{game_id}/Managers")
    _ = GameManagerResponse.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, GameManagerResponse, "GameManagerResponse")

def test_result_response_field_coverage():
    """Test that ResultResponse covers all fields returned by the API"""
    api = PyBanana()
    raw_data = api._get_data("/Util/Search/Results", params={"_sSearchString": "test"})  # Search endpoint
    _ = ResultResponse.from_dict(raw_data)  # Verify we can create the model
    return check_field_coverage(raw_data, ResultResponse, "ResultResponse")


"""Member profile and related functionality."""
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional
from datetime import datetime

from ..common.bio import Bio
from ..common.profile import Profile
from ..common.stats import CoreStats
from ..common.online import OnlineStatus
from ..common.buddy import Buddy
from ..common.field import ProfileField
from ..common.medals import Medals

@dataclass
class MemberProfile:
    base: Profile
    user_title: str = ""
    honorary_title: Optional[str] = None
    join_date: datetime = field(default_factory=lambda: datetime.fromtimestamp(0))
    avatar_url: str = ""
    upic_url: str = ""
    hd_avatar_url: str = ""
    hd_hovatar_url: str = ""
    points_url: str = ""
    medals_url: str = ""
    is_online: bool = False
    online_title: str = ""
    offline_title: str = ""
    points: int = 0
    points_rank: int = 0
    staff_profile: Optional[str] = None
    bio: List[Bio] = field(default_factory=list)
    profile_modules: List[str] = field(default_factory=list)
    is_banned: bool = False
    online_status: Optional[OnlineStatus] = None
    core_stats: Optional[CoreStats] = None
    signature_url: Optional[str] = None
    clearance_levels: List[str] = field(default_factory=list)
    responsibilities: List[str] = field(default_factory=list)
    modgroups: List[str] = field(default_factory=list)
    buddies: List[Buddy] = field(default_factory=list)
    contact_info: Optional[List[ProfileField]] = field(default_factory=list)
    pc_specs: Optional[List[ProfileField]] = field(default_factory=list)
    software_kit: Optional[List[ProfileField]] = field(default_factory=list)
    gaming_devices: Optional[List[ProfileField]] = field(default_factory=list)
    medals: List[Medals] = field(default_factory=list)

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "MemberProfile":
        base = Profile.from_dict(data)
        responsibilities = data.get("_aResponsibilities", {})   
        return cls(
            base=base,
            user_title=data.get("_sUserTitle", "") or "",
            honorary_title=data.get("_sHonoraryTitle", "") or "",
            join_date=datetime.fromtimestamp(data.get("_tsJoinDate", 0)),
            avatar_url=data.get("_sAvatarUrl", "") or "",
            upic_url=data.get("_sUpicUrl", "") or "",
            hd_avatar_url=data.get("_sHdAvatarUrl", "") or "",
            hd_hovatar_url=data.get("_sHdHovatarUrl", "") or "",
            points_url=data.get("_sPointsUrl", "") or "",
            medals_url=data.get("_sMedalsUrl", "") or "",
            is_online=data.get("_bIsOnline", False) or False,
            online_title=data.get("_sOnlineTitle", "") or "",
            offline_title=data.get("_sOfflineTitle", "") or "",
            points=data.get("_nPoints", 0) or 0,
            points_rank=data.get("_nPointsRank", 0) or 0,
            staff_profile=data.get("_sStaffProfile", "") or "",
            bio=[Bio.from_dict(bio) for bio in data.get("_aBio", [])],
            profile_modules=data.get("_aProfileModules", []),
            online_status=OnlineStatus.from_dict(data["_aOnlineStatus"]) if "_aOnlineStatus" in data else None,
            core_stats=CoreStats.from_dict(data["_aCoreStats"]) if "_aCoreStats" in data else None,
            is_banned=data.get("_bIsBanned", False) or False,
            signature_url=data.get("_sSigUrl", "") or "",
            clearance_levels=data.get("_aClearanceLevels", {}),
            responsibilities=responsibilities,
            modgroups=responsibilities.get("_aModgroups", []),
            buddies=[Buddy.from_dict(buddy) for buddy in data.get("_aBuddies", [])],
            contact_info=[ProfileField.from_dict(field) for field in data.get("_aContactInfo", [])],
            pc_specs=[ProfileField.from_dict(field) for field in data.get("_aPcSpecs", [])],
            software_kit=[ProfileField.from_dict(field) for field in data.get("_aSoftwareKit", [])],
            gaming_devices=[ProfileField.from_dict(field) for field in data.get("_aGamingDevices", [])],
            medals=[Medals.from_dict(medal) for medal in data.get("_aMedals", [])]
        )


"""Idea profile and related functionality."""
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional
from datetime import datetime

from ..common.profile import Profile
from ..common.ratings import RatingsSummary
from ..common.embeddable import Embeddable

@dataclass
class IdeaProfile:
    base: Profile  # Required field
    text: str = ""
    post_count: int = 0
    has_revisions: bool = False
    has_changelog: bool = False
    is_private: bool = False
    is_shared: bool = False
    sorting_priority: int = 0
    supports_downvoting: bool = False
    ratings_summary: Optional[RatingsSummary] = None
    embeddables: List[Embeddable] = field(default_factory=list)

    def __post_init__(self):
        if self.ratings_summary is None:
            self.ratings_summary = RatingsSummary.from_dict({})

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "IdeaProfile":
        base = Profile.from_dict(data)
        # Wrap the ratings data in the expected structure
        ratings_data = {"_aRatingsSummary": data.get("_aRatingsSummary", {})} if "_aRatingsSummary" in data else {}
        
        return cls(
            base=base,
            text=data.get("_sText", "") or "",
            post_count=data.get("_nPostCount", 0) or 0,
            has_revisions=data.get("_bHasRevisions", False),
            has_changelog=data.get("_bHasChangelog", False),
            is_private=data.get("_bIsPrivate", False),
            is_shared=data.get("_bIsShared", False),
            sorting_priority=data.get("_nSortingPriority", 0) or 0,
            supports_downvoting=data.get("_bSupportsDownvoting", False),
            ratings_summary=RatingsSummary.from_dict(ratings_data),
            embeddables=[Embeddable.from_dict(embed) for embed in data.get("_aEmbeddables", [])]
        )


"""Moderator-related shared components."""
from dataclasses import dataclass, field
from typing import Dict, Any, List, Optional

from ..member import Member

@dataclass
class ModeratorRecord:
    member: Member  # Required field
    modgroups: List[str] = field(default_factory=list)
    staff_bio: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> Optional['ModeratorRecord']:
        if not isinstance(data, dict):
            return None

        member = Member.from_dict(data.get('_aMember', {}))
        if member is None:
            return None

        return cls(
            member=member,
            modgroups=data.get('_aModgroups', []) or [],
            staff_bio=data.get('_sStaffBio', "") or ""
        )


"""License-related shared components."""
from dataclasses import dataclass, field
from typing import List, Dict, Any

@dataclass
class LicenseChecklist:
    """License checklist information."""
    yes: List[str] = field(default_factory=list)
    ask: List[str] = field(default_factory=list)
    no: List[str] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "LicenseChecklist":
        return cls(
            yes=data.get("yes", []) or [],
            ask=data.get("ask", []) or [],
            no=data.get("no", []) or []
        )


"""Rating-related shared components."""
from dataclasses import dataclass, field
from typing import Dict, Any, Optional

@dataclass
class RatingBreakdownItem:
    count: int = 0
    negativity: int = 0
    positivity: int = 0
    weight: int = 0
    
    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "RatingBreakdownItem":
        if not isinstance(data, dict):
            return cls()
            
        return cls(
            count=data.get("_nCount", 0) or 0,
            negativity=data.get("_nNegativity", 0) or 0,
            positivity=data.get("_nPositivity", 0) or 0,
            weight=data.get("_nWeight", 0) or 0
        )

@dataclass
class RatingsSummary:
    ratings_count: int = 0
    cumulative_rating: int = 0
    cumulative_positivity: int = 0
    cumulative_negativity: int = 0
    ratings_breakdown: Dict[str, RatingBreakdownItem] = field(default_factory=dict)

    def __post_init__(self):
        if self.ratings_breakdown is None:
            self.ratings_breakdown = {}

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "RatingsSummary":
        if not isinstance(data, dict):
            return cls()
            
        breakdown = {}
        breakdown_data = data.get("_aRatingsBreakdown", {}) or {}
        if isinstance(breakdown_data, dict):
            for rating_key, rating_data in breakdown_data.items():
                breakdown[rating_key] = RatingBreakdownItem.from_dict(rating_data)
                
        return cls(
            ratings_count=data.get("_nRatingsCount", 0) or 0,
            cumulative_rating=data.get("_nCumulativeRating", 0) or 0,
            cumulative_positivity=data.get("_nCumulativePositivity", 0) or 0,
            cumulative_negativity=data.get("_nCumulativeNegativity", 0) or 0,
            ratings_breakdown=breakdown
        )



"""Game profile and related functionality."""
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional
from datetime import datetime

from ..common.profile import Profile
from ..common.category import ModCategory
from ..common.managers import ManagerRecord

@dataclass
class GameSection:
    """Game section information."""
    model_name: str = ""
    plural_title: str = ""
    item_count: int = 0
    category_count: int = 0
    url: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "GameSection":
        return cls(
            model_name=data.get("_sModelName", "") or "",
            plural_title=data.get("_sPluralTitle", "") or "",
            item_count=data.get("_nItemCount", 0) or 0,
            category_count=data.get("_nCategoryCount", 0) or 0,
            url=data.get("_sUrl", "") or ""
        )

@dataclass
class GameProfile:
    """Game profile information."""
    base: Profile  # Required field - no default
    homepage: str = ""
    is_approved: bool = False 
    sections: List[GameSection] = field(default_factory=list)
    mod_categories: List[ModCategory] = field(default_factory=list)
    managers: List[ManagerRecord] = field(default_factory=list)
    abbreviation: Optional[str] = None
    release_date: Optional[datetime] = None
    welcome_message: Optional[str] = None
    description: Optional[str] = None

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "GameProfile":
        base = Profile.from_dict(data)     
        release_date = None
        if "_tsReleaseDate" in data and data["_tsReleaseDate"]:
            release_date = datetime.fromtimestamp(data["_tsReleaseDate"])
                
        return cls(
            base=base,
            homepage=data.get("_sHomepage", ""),
            is_approved=data.get("_bIsApproved", False),
            sections=[GameSection.from_dict(section) for section in data.get("_aSections", [])],
            mod_categories=[ModCategory.from_dict(cat) for cat in data.get("_aModCategories", [])],
            managers=[ManagerRecord.from_dict(m) for m in data.get("_aManagers", [])],
            abbreviation=data.get("_sAbbreviation", "") or None,
            release_date=release_date,
            welcome_message=data.get("_sWelcomeMessage", "") or "",
            description=data.get("_sDescription", "") or ""
        )


"""Base profile functionality shared by all profile types."""
from dataclasses import dataclass, field
from typing import Dict, Any, Optional
from datetime import datetime
from .preview import PreviewMedia

@dataclass
class Profile:
    """Base profile class."""
    id: Optional[int] = None
    status: Optional[int] = None
    is_private: Optional[bool] = None
    date_modified: Optional[datetime] = None
    date_added: Optional[datetime] = None
    profile_url: Optional[str] = None
    preview_media: Optional[PreviewMedia] = None
    name: Optional[str] = None
    initial_visibility: Optional[str] = None
    has_files: Optional[bool] = None
    subscriber_count: Optional[int] = None
    show_ripe_promo: Optional[bool] = None
    accessor_subscription_row: Optional[int] = None
    accessor_is_subscribed: Optional[bool] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Profile":
        preview_media_data = data.get("_aPreviewMedia", {})
        if not isinstance(preview_media_data, dict):
            preview_media_data = {}

        try:
            date_modified = datetime.fromtimestamp(data["_tsDateModified"]) if "_tsDateModified" in data else None
        except (ValueError, OSError):
            date_modified = None

        try:
            date_added = datetime.fromtimestamp(data["_tsDateAdded"]) if "_tsDateAdded" in data else None
        except (ValueError, OSError):
            date_added = None

        return cls(
            id=data.get("_idRow"),
            status=int(data.get("_nStatus", 0)) if data.get("_nStatus") is not None else None,
            is_private=data.get("_bIsPrivate"),
            date_modified=date_modified,
            date_added=date_added,
            profile_url=data.get("_sProfileUrl"),
            preview_media=PreviewMedia.from_dict(preview_media_data),
            name=data.get("_sName"),
            initial_visibility=data.get("_sInitialVisibility"),
            has_files=data.get("_bHasFiles"),
            subscriber_count=data.get("_nSubscriberCount"),
            show_ripe_promo=data.get("_bShowRipePromo"),
            accessor_subscription_row=data.get("_idAccessorSubscriptionRow"),
            accessor_is_subscribed=data.get("_bAccessorIsSubscribed")
        )


"""Stats-related shared components."""
from dataclasses import dataclass
from typing import Dict, Any, Optional

@dataclass
class CoreStats:
    account_age: str = ""
    current_submissions: int = 0
    current_subscribers: int = 0
    thanks_received: int = 0
    points: int = 0
    submissions_featured: int = 0
    medals_count: int = 0

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "CoreStats":
        return cls(
            account_age=data.get("_sAccountAge", "") or "",
            current_submissions=data.get("_nCurrentSubmissions", 0) or 0,
            current_subscribers=data.get("_nCurrentSubscribers", 0) or 0,
            thanks_received=data.get("_nThanksReceived", 0) or 0,
            points=data.get("_nPoints", 0) or 0,
            submissions_featured=data.get("_nSubmissionsFeatured", 0) or 0,
            medals_count=data.get("_nMedalsCount", 0) or 0
        )


"""Shared embeddable content functionality."""
from dataclasses import dataclass
from typing import Dict, Any, Optional, List

@dataclass
class Embeddable:
    image_base_url: str = ""
    variants: List[str] = None

    def __post_init__(self):
        if self.variants is None:
            self.variants = []

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Embeddable":
        if isinstance(data, str):
            # Handle case where data is just the image base URL
            return cls(image_base_url=data or "")
            
        return cls(
            image_base_url=data.get("_sEmbeddableImageBaseUrl", "") or "",
            variants=data.get("_aVariants", []) or []
        )


"""Club profile and related functionality."""
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional
from datetime import datetime

from ..member import Member
from ..common.profile import Profile
from ..common.preview import PreviewMedia
from ..common.managers import ManagerRecord

@dataclass
class ClubProfile:
    """Club profile information."""
    base: Profile
    status: int = 0
    is_private: bool = False
    date_modified: Optional[datetime] = None
    date_added: Optional[datetime] = None
    preview_media: Optional[PreviewMedia] = None
    accessor_is_submitter: bool = False
    is_trashed: bool = False
    name: str = ""
    post_count: int = 0
    initial_visibility: str = ""
    has_files: bool = False
    text: str = ""
    member_count: int = 0
    last_activity_date: Optional[datetime] = None
    show_ripe_promo: bool = False
    submitter: Optional[Member] = None
    flag_url: str = ""
    banner_url: str = ""
    motto: str = ""
    join_mode: str = ""
    social_links: List[str] = field(default_factory=list)
    profile_template: str = ""
    profile_modules: List[str] = field(default_factory=list)
    accessor_is_in_guild: bool = False
    accessor_has_pending_join_request: bool = False
    leadership: List[ManagerRecord] = field(default_factory=list)

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ClubProfile":
        base = Profile.from_dict(data)

        return cls(
            base=base,
            status=int(data.get("_nStatus", 0)),
            is_private=data.get("_bIsPrivate", False),
            date_modified=datetime.fromtimestamp(data.get("_tsDateModified", 0)),
            date_added=datetime.fromtimestamp(data.get("_tsDateAdded", 0)),
            preview_media=PreviewMedia.from_dict({} if isinstance(data.get("_aPreviewMedia", {}), list) else data.get("_aPreviewMedia", {})),
            accessor_is_submitter=data.get("_bAccessorIsSubmitter", False),
            is_trashed=data.get("_bIsTrashed", False),
            name=data.get("_sName", "") or "",
            post_count=data.get("_nPostCount", 0) or 0,
            initial_visibility=data.get("_sInitialVisibility", "") or "",
            has_files=data.get("_bHasFiles", False),
            text=data.get("_sText", "") or "",
            member_count=data.get("_nMemberCount", 0) or 0,
            last_activity_date=datetime.fromtimestamp(data.get("_tsLastActivityDate", 0)),
            show_ripe_promo=data.get("_bShowRipePromo", False),
            submitter=Member.from_dict(data.get("_aMember", {})) if data.get("_aMember") else None,
            flag_url=data.get("_sFlagUrl", "") or "",
            banner_url=data.get("_sBannerUrl", "") or "",
            motto=data.get("_sMotto", "") or "",
            join_mode=data.get("_sJoinMode", "") or "",
            social_links=data.get("_aSocialLinks", []),
            profile_template=data.get("_sProfileTemplate", "") or "",
            profile_modules=data.get("_aProfileModules", []),
            accessor_is_in_guild=data.get("_bAccessorIsInGuild", False),
            accessor_has_pending_join_request=data.get("_bAccessorHasPendingJoinRequest", False),
            leadership=[ManagerRecord.from_dict(leader) for leader in data.get("_aLeadership", [])]
        )


"""Mod profile and related functionality."""
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional
from datetime import datetime

from ..member import Member
from ..common.profile import Profile
from ..common.preview import PreviewMedia
from ..common.credits import Credits
from ..common.category import ModCategory
from ..common.embeddable import Embeddable
from ..common.file import File
from ..common.license import LicenseChecklist
from .studio import StudioProfile

@dataclass
class ModProfile:
    """Mod profile information."""
    base: Profile  # Required field - no default
    feedback_instructions: str = ""
    accessor_is_submitter: bool = False
    is_trashed: bool = False  
    is_withheld: bool = False
    name: str = ""
    updates_count: int = 0
    has_updates: bool = False
    all_todos_count: int = 0
    has_todos: bool = False
    post_count: int = 0
    attributes: Dict[str, List[str]] = field(default_factory=dict)
    tags: List[Dict[str, str]] = field(default_factory=list)
    created_by_submitter: bool = False
    is_ported: bool = False
    thanks_count: int = 0
    initial_visibility: str = ""
    download_url: str = ""
    download_count: int = 0
    files: List[File] = field(default_factory=list)
    subscriber_count: int = 0
    studio: Optional[StudioProfile] = None
    contributing_studios: List[Any] = field(default_factory=list)
    license: str = ""
    license_checklist: Optional[LicenseChecklist] = None
    description: str = ""
    generate_table_of_contents: bool = False
    text: str = ""
    like_count: int = 0
    view_count: int = 0
    is_mapped: bool = False
    is_textured: bool = False
    is_animated: str = ""
    accepts_donations: bool = False
    show_ripe_promo: bool = False
    embeddables: List[Embeddable] = field(default_factory=list)
    submitter: Optional[Member] = None
    category: Optional[ModCategory] = None
    credits: Optional[Credits] = None
    advanced_requirements_exist: bool = False
    requirements: List[List[str]] = field(default_factory=list)
    accessor_subscription_row_id: int = 0
    accessor_is_subscribed: bool = False
    accessor_has_thanked: bool = False
    accessor_has_unliked: bool = False
    accessor_has_liked: bool = False

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ModProfile":
        base = Profile.from_dict(data)
        return cls(
            base=base,
            feedback_instructions=data.get("_sFeedbackInstructions", ""),
            accessor_is_submitter=data.get("_bAccessorIsSubmitter", False),
            is_trashed=data.get("_bIsTrashed", False),
            is_withheld=data.get("_bIsWithheld", False),
            name=data.get("_sName", ""),
            updates_count=data.get("_nUpdatesCount", 0),
            has_updates=data.get("_bHasUpdates", False),
            all_todos_count=data.get("_nAllTodosCount", 0),
            has_todos=data.get("_bHasTodos", False),
            post_count=data.get("_nPostCount", 0),
            attributes=data.get("_aAttributes", {}),
            tags=data.get("_aTags", []),
            created_by_submitter=data.get("_bCreatedBySubmitter", False),
            is_ported=data.get("_bIsPorted", False),
            thanks_count=data.get("_nThanksCount", 0),
            initial_visibility=data.get("_sInitialVisibility", ""),
            download_url=data.get("_sDownloadUrl", ""),
            download_count=data.get("_nDownloadCount", 0),
            files=[File.from_dict(f) for f in data.get("_aFiles", [])],
            subscriber_count=data.get("_nSubscriberCount", 0),
            studio=StudioProfile.from_dict(data["_aStudio"]) if "_aStudio" in data else None,
            contributing_studios=data.get("_aContributingStudios", []),
            license=data.get("_sLicense", ""),
            license_checklist=LicenseChecklist.from_dict(data["_aLicenseChecklist"]) if "_aLicenseChecklist" in data else None,
            description=data.get("_sDescription", ""),
            generate_table_of_contents=data.get("_bGenerateTableOfContents", False),
            text=data.get("_sText", ""),
            like_count=data.get("_nLikeCount", 0),
            view_count=data.get("_nViewCount", 0),
            is_mapped=data.get("_bIsMapped", False),
            is_textured=data.get("_bIsTextured", False),
            is_animated=data.get("_xIsAnimated", ""),
            accepts_donations=data.get("_bAcceptsDonations", False),
            show_ripe_promo=data.get("_bShowRipePromo", False),
            embeddables=[Embeddable.from_dict(item) for item in data.get("_aEmbeddables", [])],
            submitter=Member.from_dict(data["_aSubmitter"]) if "_aSubmitter" in data else None,
            category=ModCategory.from_dict(data["_aCategory"]) if "_aCategory" in data else None,
            credits=Credits.from_dict(data["_aCredits"]) if "_aCredits" in data else None,
            advanced_requirements_exist=data.get("_bAdvancedRequirementsExist", False),
            requirements=data.get("_aRequirements", []),
            accessor_subscription_row_id=data.get("_idAccessorSubscriptionRow", 0),
            accessor_is_subscribed=data.get("_bAccessorIsSubscribed", False),
            accessor_has_thanked=data.get("_bAccessorHasThanked", False),
            accessor_has_unliked=data.get("_bAccessorHasUnliked", False),
            accessor_has_liked=data.get("_bAccessorHasLiked", False)
        )



"""API response models for GameBanana endpoints."""
from dataclasses import dataclass, field
from typing import Dict, Any, List, Optional
from ..result import Result
from .moderators import ModeratorRecord
from .managers import ManagerRecord
from .online import OnlineRecord
from .discussion import DiscussionRecord

@dataclass
class ModeratorResponse:
    """Response from the moderators endpoint."""
    metadata: Dict[str, Any] = field(default_factory=dict)
    records: List[ModeratorRecord] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'ModeratorResponse':
        records = [rec for rec in (ModeratorRecord.from_dict(record) for record in data.get('_aRecords', [])) if rec is not None]
        return cls(
            metadata=data.get("_aMetadata", {}),
            records=records
        )

@dataclass
class GameManagerResponse:
    """Response from the game managers endpoint."""
    metadata: Dict[str, Any] = field(default_factory=dict)
    records: List[ManagerRecord] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: List[Dict[str, Any]] | Dict[str, Any]) -> 'GameManagerResponse':
        """Create a GameManagerResponse instance from either a list or object format."""
        if isinstance(data, list):
            records = [rec for rec in (ManagerRecord.from_dict(record) for record in data) if rec is not None]
            return cls(records=records)
        
        records = [rec for rec in (ManagerRecord.from_dict(record) for record in data.get('_aRecords', [])) if rec is not None]
        return cls(
            metadata=data.get("_aMetadata", {}),
            records=records
        )

@dataclass
class ResultResponse:
    """Container for search results that matches the GameBanana API response structure."""
    metadata: Dict[str, Any] = field(default_factory=dict)
    records: List[Result] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'ResultResponse':
        """Create a ResultList instance from a dictionary."""
        return cls(
            metadata=data.get("_aMetadata", {}),
            records=[Result.from_dict(record) for record in data["_aRecords"]],
        )
    
@dataclass
class OnlineResponse:
    """Container for online status that matches the GameBanana API response structure."""
    metadata: Dict[str, Any] = field(default_factory=dict)
    records: List[OnlineRecord] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'OnlineResponse':
        """Create an OnlineResponse instance from a dictionary."""
        records = [rec for rec in (OnlineRecord.from_dict(record) for record in data.get('_aRecords', [])) if rec is not None]
        return cls(
            metadata=data.get("_aMetadata", {}),
            records=records
        )
    
@dataclass
class DiscussionResponse:
    """Container for discussion records that matches the GameBanana API response structure."""
    metadata: Dict[str, Any] = field(default_factory=dict)
    records: List[DiscussionRecord] = field(default_factory=list)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> 'DiscussionResponse':
        """Create a DiscussionResponse instance from a dictionary."""
        records = [rec for rec in (DiscussionRecord.from_dict(record) for record in data.get('_aRecords', [])) if rec is not None]
        return cls(
            metadata=data.get("_aMetadata", {}),
            records=records
        )


from setuptools import setup, find_packages
import os

# Read the content of README.md for the long description
with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="pybanana",
    version="0.4.5",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    install_requires=[
        "requests>=2.25.0",
        "python-dateutil>=2.8.2",
    ],
    extras_require={
        "dev": [
            "pytest>=8.0.0",
            "requests-mock>=1.11.0",
        ],
    },
    author="robin",
    author_email="xrobinsong@gmail.com",
    description="A Python wrapper for the GameBanana API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/robinxoxo/pybanana",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)


"""Category-related shared components."""
from dataclasses import dataclass
from typing import Dict, Optional, Any

@dataclass
class ModCategory:
    id: int = 0
    name: str = ""
    item_count: int = 0
    category_count: int = 0
    url: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "ModCategory":
        if not isinstance(data, dict):
            return cls()

        return cls(
            id=data.get("_idRow", 0) or 0,
            name=data.get("_sName", "") or "",
            item_count=data.get("_nItemCount", 0) or 0,
            category_count=data.get("_nCategoryCount", 0) or 0,
            url=data.get("_sUrl", "") or ""
        )


"""Discussion record models."""
from dataclasses import dataclass, field
from typing import Dict, Any, Optional

from ..member import Member
from .profile import Profile
from .category import ModCategory
from ..profiles.game import GameSection

@dataclass
class Submission:
    """Submission information in a discussion record."""
    base: Profile
    model_name: Optional[str] = None
    singular_title: Optional[str] = None
    icon_classes: Optional[str] = None
    date_updated: Optional[int] = None
    submitter: Optional[Member] = None
    game: Optional[GameSection] = None
    root_category: Optional[ModCategory] = None
    version: Optional[str] = None
    is_obsolete: bool = False
    has_content_ratings: bool = False
    like_count: int = 0
    post_count: int = 0
    was_featured: bool = False
    view_count: int = 0
    is_owned_by_accessor: bool = False

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Submission":
        if not data:
            return None
        
        return cls(
            base=Profile.from_dict(data),
            model_name=data.get("_sModelName"),
            singular_title=data.get("_sSingularTitle"),
            icon_classes=data.get("_sIconClasses"),
            date_updated=data.get("_tsDateUpdated"),
            submitter=Member.from_dict(data.get("_aSubmitter")) if data.get("_aSubmitter") else None,
            game=GameSection.from_dict(data.get("_aGame")) if data.get("_aGame") else None,
            root_category=ModCategory.from_dict(data.get("_aRootCategory")) if data.get("_aRootCategory") else None,
            version=data.get("_sVersion"),
            is_obsolete=data.get("_bIsObsolete", False),
            has_content_ratings=data.get("_bHasContentRatings", False),
            like_count=data.get("_nLikeCount", 0) or 0,
            post_count=data.get("_nPostCount", 0) or 0,
            was_featured=data.get("_bWasFeatured", False),
            view_count=data.get("_nViewCount", 0) or 0,
            is_owned_by_accessor=data.get("_bIsOwnedByAccessor", False)
        )

@dataclass
class Post:
    """Post information in a discussion record."""
    id: Optional[int] = None
    date_added: Optional[int] = None
    text: Optional[str] = None
    poster: Optional[Member] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "Post":
        if not data:
            return None
        return cls(
            id=data.get("_idRow", 0) or 0,
            date_added=data.get("_tsDateAdded", 0) or 0,
            text=data.get("_sText", "") or "",
            poster=Member.from_dict(data.get("_aPoster", {})) if data.get("_aPoster") else None
        )

@dataclass
class DiscussionRecord:
    """A discussion record containing a submission and a post."""
    submission: Optional[Submission] = None
    post: Optional[Post] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "DiscussionRecord":
        """Create a DiscussionRecord from dictionary data."""
        if not data:
            return cls()
            
        return cls(
            submission=Submission.from_dict(data.get("_aSubmission")) if data.get("_aSubmission") else None,
            post=Post.from_dict(data.get("_aPost")) if data.get("_aPost") else None
        )



import pytest
from datetime import datetime
from pybanana.models.member import Member, Moderator
from pybanana.models.common.profile import Profile
from pybanana.models.profiles.mod import ModProfile
from pybanana.models.profiles.game import GameProfile, GameSection
from pybanana.models.profiles.app import AppProfile, AppFeatures
from pybanana.models.common.preview import PreviewMedia, PreviewMediaImage
from pybanana.models.common.ratings import RatingsSummary, RatingBreakdownItem
from pybanana.models.common.file import File
from pybanana.models.profiles.idea import IdeaProfile
from pybanana.models.profiles.club import ClubProfile
from pybanana.models.profiles.bug import BugProfile
from pybanana.models.profiles.studio import StudioProfile, OpenPosition
from pybanana.models.common.stats import CoreStats
from pybanana.models.common.license import LicenseChecklist
from pybanana.models.common.bio import Bio, BioEntry
from pybanana.models.common.credits import Author, CreditGroup, AffiliatedStudio, Credits
from pybanana.models.common.embeddable import Embeddable
from pybanana.models.common.online import OnlineStatus
from pybanana.models.common.managers import ManagerRecord
from pybanana.models.common.moderators import ModeratorRecord
from pybanana.models.common.category import ModCategory

def test_empty_member():
    """Test Member with empty data should have default values"""
    empty_data = {}
    member = Member.from_dict(empty_data)
    assert member.id == 0
    assert member.name == ""
    assert member.is_online is False
    assert member.has_ripe is False
    assert member.profile_url == ""
    assert member.avatar_url == ""

def test_empty_profile():
    """Test Profile with empty data should have default values"""
    empty_data = {}
    profile = Profile.from_dict(empty_data)
    assert profile.id is None
    assert profile.status is None
    assert profile.is_private is None
    assert profile.date_modified is None
    assert profile.date_added is None
    assert profile.profile_url is None
    assert profile.preview_media is None
    assert profile.name is None

def test_empty_mod_profile():
    """Test ModProfile with empty data"""
    empty_data = {}
    profile = ModProfile.from_dict(empty_data)
    assert profile.feedback_instructions == ""
    assert profile.accessor_is_submitter is False
    assert profile.is_trashed is False
    assert profile.is_withheld is False
    assert profile.name == ""
    assert profile.updates_count == 0

def test_empty_game_profile():
    """Test GameProfile with empty data"""
    empty_data = {}
    profile = GameProfile.from_dict(empty_data)
    assert profile.homepage == ""
    assert profile.is_approved is False
    assert profile.sections == []
    assert profile.mod_categories == []
    assert profile.managers == []

def test_empty_app_profile():
    """Test AppProfile with empty data"""
    empty_data = {}
    profile = AppProfile.from_dict(empty_data)
    assert profile.description == ""
    assert profile.include_variable_name == ""
    assert profile.version == ""
    assert profile.state == ""
    assert profile.type == ""
    assert profile.user_count == 0
    assert profile.is_safe is False
    assert profile.accepts_donations is False
    assert profile.accessor_has_app is False
    assert isinstance(profile.features, AppFeatures)

    # Test with minimum required fields
    min_data = {
        "_sDescription": "",
        "_sIncludeVariableName": "",
        "_sVersion": "",
        "_sState": "",
        "_sType": "",
        "_nUserCount": 0,
        "_sbIsSafe": "false",
        "_bAcceptsDonations": False,
        "_bAccessorHasApp": False
    }
    profile = AppProfile.from_dict(min_data)
    assert profile.description == ""
    assert profile.include_variable_name == ""
    assert profile.version == ""
    assert profile.state == ""
    assert profile.type == ""
    assert profile.user_count == 0
    assert profile.is_safe is False
    assert profile.accepts_donations is False
    assert profile.accessor_has_app is False

def test_empty_ratings_summary():
    """Test RatingsSummary with empty data"""
    empty_data = {}
    summary = RatingsSummary.from_dict(empty_data)
    assert summary.ratings_count == 0
    assert summary.cumulative_rating == 0
    assert summary.cumulative_positivity == 0
    assert summary.cumulative_negativity == 0
    assert isinstance(summary.ratings_breakdown, dict)
    assert len(summary.ratings_breakdown) == 0

def test_empty_file():
    """Test File with empty data"""
    empty_data = {}
    file = File.from_dict(empty_data)
    assert file.id == 0
    assert file.filename == ""
    assert file.filesize == 0
    assert file.description == ""
    assert isinstance(file.date_added, datetime)
    assert file.download_count == 0
    assert file.analysis_state == ""
    assert file.analysis_result_code == ""
    assert file.analysis_result == ""
    assert file.contains_exe is False
    assert file.download_url == ""
    assert file.md5_checksum == ""
    assert file.clam_av_result == ""
    assert file.avast_av_result == ""

def test_empty_idea_profile():
    """Test IdeaProfile with empty data"""
    empty_data = {}
    profile = IdeaProfile.from_dict(empty_data)
    assert profile.text == ""
    assert profile.post_count == 0
    assert profile.has_revisions is False
    assert profile.has_changelog is False
    assert profile.is_private is False
    assert profile.is_shared is False
    assert profile.sorting_priority == 0
    assert profile.supports_downvoting is False
    assert isinstance(profile.ratings_summary, RatingsSummary)
    assert isinstance(profile.embeddables, list)
    assert len(profile.embeddables) == 0

def test_empty_club_profile():
    """Test ClubProfile with empty data"""
    empty_data = {}
    profile = ClubProfile.from_dict(empty_data)
    assert profile.status == 0
    assert profile.is_private is False
    assert isinstance(profile.date_modified, datetime)
    assert isinstance(profile.date_added, datetime)
    assert isinstance(profile.preview_media, PreviewMedia)
    assert profile.accessor_is_submitter is False
    assert profile.is_trashed is False
    assert profile.name == ""
    assert profile.post_count == 0
    assert profile.initial_visibility == ""
    assert profile.has_files is False
    assert profile.text == ""
    assert profile.member_count == 0
    assert isinstance(profile.last_activity_date, datetime)
    assert profile.show_ripe_promo is False
    assert profile.flag_url == ""
    assert profile.banner_url == ""
    assert profile.motto == ""
    assert profile.join_mode == ""
    assert isinstance(profile.social_links, list)
    assert len(profile.social_links) == 0
    assert profile.profile_template == ""
    assert isinstance(profile.profile_modules, list)
    assert len(profile.profile_modules) == 0
    assert profile.accessor_is_in_guild is False
    assert profile.accessor_has_pending_join_request is False
    assert isinstance(profile.leadership, list)
    assert len(profile.leadership) == 0

def test_empty_bug_profile():
    """Test BugProfile with empty data"""
    empty_data = {}
    # BugProfile should handle empty data with defaults
    profile = BugProfile.from_dict(empty_data)
    assert profile.status == 0
    assert profile.is_private is False
    assert isinstance(profile.date_modified, datetime)
    assert isinstance(profile.date_added, datetime)
    assert profile.preview_media is not None
    assert profile.accessor_is_submitter is False
    assert profile.is_trashed is False
    assert profile.post_count == 0
    assert profile.thanks_count == 0
    assert profile.initial_visibility == ""
    assert profile.has_files is False
    assert profile.subscriber_count == 0
    assert profile.text == ""
    assert profile.resolution == ""
    assert profile.resolution_key == ""
    assert profile.priority == ""
    assert profile.priority_key == ""
    assert profile.source_url == ""
    assert profile.show_ripe_promo is False
    assert profile.embeddables == []
    assert profile.submitter is None
    assert profile.accessor_subscription_row_id == 0
    assert profile.accessor_is_subscribed is False
    assert profile.accessor_has_thanked is False

    # Test with minimum required fields
    min_data = {
        "_nStatus": 0,
        "_bIsPrivate": False,
        "_tsDateModified": 0,
        "_tsDateAdded": 0,
        "_aPreviewMedia": {"_aImages": [], "_aMetadata": {}},
        "_bAccessorIsSubmitter": False,
        "_bIsTrashed": False,
        "_nPostCount": 0,
        "_nThanksCount": 0,
        "_sInitialVisibility": "",
        "_bHasFiles": False,
        "_nSubscriberCount": 0,
        "_sText": "",
        "_sResolution": "",
        "_sResolutionKey": "",
        "_sPriority": "",
        "_sPriorityKey": "",
        "_sSourceUrl": "",
        "_bShowRipePromo": False,
        "_aEmbeddables": [],
        "_aMember": {
            "_idRow": 0,
            "_sName": "",
            "_bIsOnline": False,
            "_bHasRipe": False,
            "_sProfileUrl": "",
            "_sAvatarUrl": ""
        }
    }
    profile = BugProfile.from_dict(min_data)
    assert profile.status == 0
    assert profile.is_private is False
    assert isinstance(profile.date_modified, datetime)
    assert isinstance(profile.date_added, datetime)

def test_empty_studio_profile():
    """Test StudioProfile with empty data"""
    empty_data = {}
    profile = StudioProfile.from_dict(empty_data)
    assert profile.motto is None
    assert profile.join_mode == ""
    assert profile.flag_url is None
    assert profile.banner_url is None
    assert profile.social_links == []
    assert profile.open_positions == []
    assert profile.leadership == []

def test_empty_preview_media():
    """Test PreviewMedia with empty data"""
    empty_data = {}
    # PreviewMedia should initialize with empty lists
    media = PreviewMedia.from_dict(empty_data)
    assert media.images == []
    assert media.metadata == {}

def test_empty_core_stats():
    """Test CoreStats with empty data"""
    empty_data = {}
    stats = CoreStats.from_dict(empty_data)
    assert stats.account_age == ""
    assert stats.current_submissions == 0
    assert stats.current_subscribers == 0
    assert stats.thanks_received == 0
    assert stats.points == 0
    assert stats.submissions_featured == 0
    assert stats.medals_count == 0

def test_empty_license_checklist():
    """Test LicenseChecklist with empty data"""
    empty_data = {}
    checklist = LicenseChecklist.from_dict(empty_data)
    assert checklist.yes == []
    assert checklist.ask == []
    assert checklist.no == []

def test_empty_bio_entry():
    """Test BioEntry with various empty, None, and missing field scenarios"""
    # Test with empty data
    empty_data = {}
    entry = BioEntry.from_dict(empty_data)
    assert entry.title == ""
    assert entry.value == ""
    assert entry.custom_title == ""

    # Test with None values
    none_data = {
        "_sTitle": None,
        "_sValue": None,
        "_sCustomTitle": None
    }
    entry = BioEntry.from_dict(none_data)
    assert entry.title == ""
    assert entry.value == ""
    assert entry.custom_title == ""

    # Test with missing fields
    partial_data = {
        "_sTitle": "Test Title"
        # _sValue and _sCustomTitle are missing
    }
    entry = BioEntry.from_dict(partial_data)
    assert entry.title == "Test Title"
    assert entry.value == ""
    assert entry.custom_title == ""

def test_empty_bio():
    """Test Bio with various empty, None, and missing field scenarios"""
    # Test with empty data
    empty_data = {}
    bio = Bio.from_dict(empty_data)
    assert bio.entries == []

    # Test with None _aBio
    none_data = {"_aBio": None}
    bio = Bio.from_dict(none_data)
    assert bio.entries == []

    # Test with empty _aBio list
    empty_list_data = {"_aBio": []}
    bio = Bio.from_dict(empty_list_data)
    assert bio.entries == []

    # Test with direct bio entry format
    direct_entry = {
        "_sTitle": "Direct Title",
        "_sValue": "Direct Value"
    }
    bio = Bio.from_dict(direct_entry)
    assert len(bio.entries) == 1
    assert bio.entries[0].title == "Direct Title"
    assert bio.entries[0].value == "Direct Value"

    # Test with mixed valid and invalid entries
    mixed_data = {
        "_aBio": [
            {
                "_sTitle": "Valid Title",
                "_sValue": "Valid Value"
            },
            {},  # Empty entry
            {
                "_sTitle": None,  # None values
                "_sValue": None
            },
            {
                "_sTitle": "Partial"  # Missing value
            }
        ]
    }
    bio = Bio.from_dict(mixed_data)
    assert len(bio.entries) == 4
    assert bio.entries[0].title == "Valid Title"
    assert bio.entries[0].value == "Valid Value"
    assert bio.entries[1].title == ""
    assert bio.entries[1].value == ""
    assert bio.entries[2].title == ""
    assert bio.entries[2].value == ""
    assert bio.entries[3].title == "Partial"
    assert bio.entries[3].value == ""

def test_empty_credit_group():
    """Test CreditGroup with empty data"""
    empty_data = {}
    group = CreditGroup.from_dict(empty_data)
    assert group.group_name == ""
    assert isinstance(group.authors, list)
    assert len(group.authors) == 0

def test_empty_affiliated_studio():
    """Test AffiliatedStudio with empty data"""
    empty_data = {}
    studio = AffiliatedStudio.from_dict(empty_data)
    assert studio.profile_url == ""
    assert studio.name == ""
    assert studio.flag_url == ""
    assert studio.banner_url == ""

def test_empty_author():
    """Test Author with empty data"""
    empty_data = {}
    author = Author.from_dict(empty_data)
    assert author.role == ""
    assert author.name == ""
    assert author.id == 0
    assert author.upic_url == ""
    assert author.profile_url == ""
    assert author.is_online is False
    assert author.affiliated_studio is None

def test_empty_open_position():
    """Test OpenPosition with empty data"""
    empty_data = {}
    position = OpenPosition.from_dict(empty_data)
    assert position.skill_id == ""
    assert position.skill == ""
    assert position.game_id == ""
    assert isinstance(position.game, dict)
    assert position.game == {"name": "", "url": ""}
    assert position.notes == ""

def test_empty_game_section():
    """Test GameSection with empty data"""
    empty_data = {}
    section = GameSection.from_dict(empty_data)
    assert section.plural_title == ""
    assert section.item_count == 0
    assert section.category_count == 0
    assert section.url == ""

def test_empty_app_features():
    """Test AppFeatures with empty data"""
    empty_data = {}
    features = AppFeatures.from_dict(empty_data)
    assert features.profile_module_url == ""
    assert features.navigator_tab_url == ""
    assert features.main_url == ""
    assert features.settings_url == ""

def test_empty_preview_media_image():
    """Test PreviewMediaImage with empty data"""
    empty_data = {}
    image = PreviewMediaImage.from_dict(empty_data)
    assert image.url == ""
    assert image.width is None
    assert image.height is None 
    assert image.file_size is None

def test_empty_online_status():
    """Test OnlineStatus with empty data"""
    empty_data = {}
    status = OnlineStatus.from_dict(empty_data)
    assert status.is_online is False
    assert status.location == ""
    assert isinstance(status.last_seen_time, datetime)
    assert isinstance(status.session_creation_time, datetime)

def test_empty_manager_record():
    """Test ManagerRecord with empty data"""
    empty_data = {}
    manager = ManagerRecord.from_dict(empty_data)
    assert isinstance(manager.member, Member)
    assert manager.modgroups == []
    assert manager.date_added is None

def test_empty_mod_category():
    """Test ModCategory with empty data"""
    empty_data = {}
    category = ModCategory.from_dict(empty_data)
    assert category.id == 0
    assert category.name == ""
    assert category.item_count == 0
    assert category.category_count == 0
    assert category.url == ""

    # Test with None values
    none_data = {
        "_idRow": None,
        "_sName": None,
        "_nItemCount": None,
        "_nCategoryCount": None,
        "_sUrl": None
    }
    category = ModCategory.from_dict(none_data)
    assert category.id == 0
    assert category.name == ""
    assert category.item_count == 0
    assert category.category_count == 0
    assert category.url == ""

    # Test with mixed values
    mixed_data = {
        "_idRow": 1,
        "_sName": None,
        "_nItemCount": 5,
        # _nCategoryCount missing
        "_sUrl": None
    }
    category = ModCategory.from_dict(mixed_data)
    assert category.id == 1
    assert category.name == ""
    assert category.item_count == 5
    assert category.category_count == 0
    assert category.url == ""

def test_empty_embeddable():
    """Test Embeddable with empty data"""
    empty_data = {}
    embeddable = Embeddable.from_dict(empty_data)
    assert embeddable.image_base_url == ""
    assert isinstance(embeddable.variants, list)
    assert len(embeddable.variants) == 0

def test_empty_moderator_record():
    """Test ModeratorRecord with empty data"""
    empty_data = {}
    moderator = ModeratorRecord.from_dict(empty_data)
    assert isinstance(moderator.member, Member)
    assert moderator.staff_bio == ""
    assert moderator.modgroups == []

def test_empty_rating_breakdown_item():
    """Test RatingBreakdownItem with empty data"""
    empty_data = {}
    item = RatingBreakdownItem.from_dict(empty_data)
    assert item.count == 0
    assert item.verb == ""
    assert item.icon_url == ""
    assert item.icon_classes == ""
    
def test_empty_moderator():
    """Test Moderator with empty data"""
    empty_data = {}
    moderator = Moderator.from_dict(empty_data)
    assert moderator.id == 0
    assert moderator.name == ""
    assert moderator.is_online is False
    assert moderator.has_ripe is False
    assert moderator.profile_url == ""
    assert moderator.avatar_url == ""


"""Studio profile and related functionality."""
from dataclasses import dataclass, field
from typing import List, Dict, Any, Optional
from datetime import datetime

from ..member import Member
from ..common.profile import Profile
from ..common.preview import PreviewMedia
from ..common.category import ModCategory

@dataclass
class OpenPosition:
    """Open position in a studio."""
    skill_id: str = ""
    skill: str = ""
    game_id: str = ""
    game: Dict[str, str] = field(default_factory=lambda: {"name": "", "url": ""})
    notes: str = ""

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "OpenPosition":
        return cls(
            skill_id=data.get("_idSkillRow", "") or "",
            skill=data.get("_sSkill", "") or "",
            game_id=data.get("_idGameRow", "") or "",
            game=data.get("_aGame") or {"name": "", "url": ""},
            notes=data.get("_sNotes", "") or ""
        )

@dataclass
class StudioProfile:
    """Studio profile information."""
    base: Profile
    motto: Optional[str] = None
    join_mode: str = ""
    flag_url: Optional[str] = None
    banner_url: Optional[str] = None
    member_count: int = 0
    post_count: int = 0
    social_links: List[Dict[str, str]] = field(default_factory=list)
    profile_template: str = ""
    profile_modules: List[str] = field(default_factory=list)
    open_positions: List[OpenPosition] = field(default_factory=list)
    leadership: List[Member] = field(default_factory=list)
    accessor_is_in_guild: bool = False
    accessor_has_pending_join_request: bool = False
    game: Dict[str, str] = field(default_factory=dict)  # Changed to use field(default_factory=dict)

    def __getattr__(self, name):
        """Delegate attribute access to base Profile."""
        return getattr(self.base, name)

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> "StudioProfile":
        base = Profile.from_dict(data)
        return cls(
            base=base,
            motto=data.get("_sMotto"),
            join_mode=data.get("_sJoinMode", ""),
            flag_url=data.get("_sFlagUrl"),
            banner_url=data.get("_sBannerUrl"),
            member_count=data.get("_iMemberCount", 0),  # Added default
            post_count=data.get("_nPostCount", 0),  # Added default
            social_links=data.get("_aSocialLinks", []),
            profile_template=data.get("_sProfileTemplate", ""),
            profile_modules=data.get("_aProfileModules", []),
            open_positions=[OpenPosition.from_dict(pos) for pos in data.get("_aOpenPositions", [])],
            leadership=[member for member in (Member.from_dict(m) for m in data.get("_aLeadership", [])) if member is not None],
            accessor_is_in_guild=data.get("_bAccessorIsInGuild", False),
            accessor_has_pending_join_request=data.get("_bAccessorHasPendingJoinRequest", False)
        )


from enum import Enum

class OrderResult(str, Enum):
    """Enum for search order options."""
    RELEVANCE = "best_match"
    POPULARITY = "popularity"
    NEWEST = "date"
    UPDATED = "udate"

class ModelType(str, Enum):
    """Enum for model names used in the GameBanana API."""
    APP = "App"
    ARTICLE = "Article"
    BUG = "Bug"
    BLOG = "Blog"
    CLUB = "Club"
    CONTEST = "Contest"
    CONCEPT = "Concept"
    EVENT = "Event"
    GAME = "Game"
    IDEA = "Idea"
    INITIATIVE = "Initiative"
    JAM = "Jam"
    MOD = "Mod"
    MODEL = "Model"
    MEMBER = "Member"
    NEWS = "News"
    POLL = "Poll"
    PROJECT = "Project"
    QUESTION = "Question"
    REVIEW = "Review"
    REQUEST = "Request"
    SCRIPT = "Script"
    SOUND = "Sound"
    SPRAY = "Spray"
    STUDIO = "Studio"
    THREAD = "Thread"
    TOOL = "Tool"
    TUTORIAL = "Tutorial"
    WIKI = "Wiki"
    WIP = "Wip"


"""Manager-related shared components."""
from dataclasses import dataclass, field
from typing import Dict, Any, List, Optional
from datetime import datetime

from ..member import Member

@dataclass
class ManagerRecord:
    member: Member  # Required field
    modgroups: List[str] = field(default_factory=list)
    date_added: Optional[datetime] = None

    @classmethod
    def from_dict(cls, data: Dict[str, Any]) -> Optional['ManagerRecord']:
        member = Member.from_dict(data.get('_aMember', {}))
        if member is None:
            return None

        try:
            timestamp = data.get('_tsDateAdded', 0)
            date_added = datetime.fromtimestamp(timestamp) if timestamp else None
        except (ValueError, OSError):
            date_added = None

        return cls(
            member=member,
            modgroups=data.get('_aModgroups', []) or [],
            date_added=date_added
        )



"""
PyBanana - A Python wrapper for the GameBanana API.
"""

# Package metadata
__version__ = "0.4.5"
__author__ = "robin"
__license__ = "MIT"

# Main API class
from .api import PyBanana

# Import all models from the models package
from .models import (
    # Base models
    Member, Moderator, Manager,
    Buddy, SubjectShaper,
    Profile,
    PreviewMedia, PreviewMediaImage,
    ModCategory,
    
    # Common components
    Author, CreditGroup, AffiliatedStudio, Credits,
    Embeddable,
    RatingsSummary, RatingBreakdownItem,
    CoreStats,
    Bio, BioEntry,
    ProfileField, ContactInfo, PcSpecs, SoftwareKit, GamingDevices,
    File,
    LicenseChecklist,
    Medals,
    OnlineStatus,
    ManagerRecord, 
    ModeratorRecord,
    DiscussionRecord,
    
    # Profile models
    MemberProfile,
    ModProfile,
    GameProfile, GameSection,
    AppProfile, AppFeatures,
    StudioProfile, OpenPosition,
    ClubProfile,
    BugProfile,
    IdeaProfile,
    
    # Response models
    ModeratorResponse,
    GameManagerResponse,
    ResultResponse,
    OnlineResponse,
    DiscussionResponse
)

from .models.result import Result

# Define what should be imported with "from pybanana import *"
__all__ = [
    "PyBanana",
    # Base models
    "Member", "Moderator", "Manager",
    "Buddy", "SubjectShaper",
    "Profile",
    "PreviewMedia", "PreviewMediaImage",
    "ModCategory",
    
    # Common components
    "Author", "CreditGroup", "AffiliatedStudio", "Credits",
    "Embeddable",
    "RatingsSummary", "RatingBreakdownItem",
    "CoreStats",
    "Bio", "BioEntry",
    "ProfileField", "ContactInfo", "PcSpecs", "SoftwareKit", "GamingDevices",
    "File",
    "LicenseChecklist",
    "Medals",
    "OnlineStatus",
    "ManagerRecord", 
    "ModeratorRecord",
    "DiscussionRecord",
    
    # Profile models
    "MemberProfile",
    "ModProfile",
    "GameProfile", "GameSection",
    "AppProfile", "AppFeatures",
    "StudioProfile", "OpenPosition",
    "ClubProfile", 
    "BugProfile",
    "IdeaProfile",
    
    # Response models
    "ModeratorResponse",
    "GameManagerResponse",
    "ResultResponse",
    "OnlineResponse",
    "DiscussionResponse",
    
    # Additional models
    "Result",
]


import pytest
from datetime import datetime
from pybanana.models.member import Member, Moderator
from pybanana.models.common.buddy import Buddy, SubjectShaper
from pybanana.models.common.managers import ManagerRecord
from pybanana.models.common.moderators import ModeratorRecord
from pybanana.models.common.credits import Credits, Author, CreditGroup, AffiliatedStudio
from pybanana.models.common.profile import Profile
from pybanana.models.profiles.member import MemberProfile
from pybanana.models.profiles.mod import ModProfile
from pybanana.models.common.online import OnlineStatus
from pybanana.models.common.stats import CoreStats
from pybanana.models.common.bio import Bio
from pybanana.models.common.preview import PreviewMedia, PreviewMediaImage
from pybanana.models.common.license import LicenseChecklist
from pybanana.models.profiles.game import GameSection, GameProfile
from pybanana.models.profiles.app import AppFeatures, AppProfile
from pybanana.models.profiles.studio import OpenPosition, StudioProfile
from pybanana.models.profiles.club import ClubProfile
from pybanana.models.common.responses import ModeratorResponse, GameManagerResponse, ResultResponse
from pybanana.models.common.ratings import RatingsSummary
from pybanana.models.profiles.bug import BugProfile
from pybanana.models.profiles.idea import IdeaProfile
from pybanana.models.common.embeddable import Embeddable
from pybanana.models.common.category import ModCategory
from pybanana.models.common.file import File
from pybanana.models.result import Result

def test_manager_record():
    # Test data using actual API field names
    test_data = {
        '_aMember': {
            '_idRow': 123,
            '_sName': 'TestUser',
            '_bIsOnline': True,
            '_bHasRipe': False,
            '_sProfileUrl': 'https://gamebanana.com/members/123',
            '_sAvatarUrl': 'https://gamebanana.com/avatars/123.jpg'
        },
        '_aModgroups': ['admin', 'moderator'],
        '_tsDateAdded': 1234567890
    }

    # Create ManagerRecord instance
    record = ManagerRecord.from_dict(test_data)

    # Assert record data
    assert isinstance(record, ManagerRecord)
    assert isinstance(record.member, Member)
    assert record.member.id == 123
    assert record.member.name == 'TestUser'
    assert record.member.is_online is True
    assert record.modgroups == ['admin', 'moderator']
    assert record.date_added == 1234567890

def test_moderator_record():
    test_data = {
        '_aMember': {
            '_idRow': 456,
            '_sName': 'ModUser',
            '_bIsOnline': False,
            '_bHasRipe': True,
            '_sProfileUrl': 'https://gamebanana.com/members/456',
            '_sAvatarUrl': 'https://gamebanana.com/avatars/456.jpg'
        },
        '_sStaffBio': 'Test staff bio',
        '_aModgroups': ['admin', 'moderator', 'support']
    }

    record = ModeratorRecord.from_dict(test_data)

    # Assert record data
    assert isinstance(record, ModeratorRecord)
    assert isinstance(record.member, Member)
    assert record.member.id == 456
    assert record.member.name == 'ModUser'
    assert record.member.is_online is False
    assert record.staff_bio == 'Test staff bio'
    assert record.modgroups == ['admin', 'moderator', 'support']

def test_credits():
    test_data = {
        '_aGroups': [{
            '_sGroupName': 'Development Team',
            '_aAuthors': [
                {
                    '_sRole': 'Developer',
                    '_idRow': 789,
                    '_sName': 'CreditUser',
                    '_sUpicUrl': 'https://example.com/pic.jpg',
                    '_sProfileUrl': 'https://gamebanana.com/members/789',
                    '_bIsOnline': True
                },
                {
                    '_sRole': 'Artist',
                    '_idRow': 790,
                    '_sName': 'ArtistUser',
                    '_sUpicUrl': None,
                    '_sProfileUrl': 'https://gamebanana.com/members/790',
                    '_bIsOnline': False
                }
            ]
        }]
    }

    credits = Credits.from_dict(test_data)

    assert len(credits.groups) == 1
    group = credits.groups[0]
    assert isinstance(group, CreditGroup)
    assert group.group_name == 'Development Team'
    assert len(group.authors) == 2
    
    # Test first author
    author1 = group.authors[0]
    assert isinstance(author1, Author)
    assert author1.id == 789
    assert author1.name == 'CreditUser'
    assert author1.role == 'Developer'
    assert author1.upic_url == 'https://example.com/pic.jpg'
    assert author1.profile_url == 'https://gamebanana.com/members/789'
    assert author1.is_online is True

    # Test second author
    author2 = group.authors[1]
    assert isinstance(author2, Author)
    assert author2.id == 790
    assert author2.name == 'ArtistUser'
    assert author2.role == 'Artist'
    assert author2.upic_url == ''  # Default empty string when None
    assert author2.profile_url == 'https://gamebanana.com/members/790'
    assert author2.is_online is False

def test_credits_empty():
    credits = Credits.from_dict([])
    assert len(credits.groups) == 0

    credits = Credits.from_dict(None)
    assert len(credits.groups) == 0

def test_profile():
    test_data = {
        '_idRow': 101,
        '_nStatus': 1,
        '_bIsPrivate': False,
        '_tsDateModified': 1234567890,
        '_tsDateAdded': 1234567880,
        '_sProfileUrl': 'https://gamebanana.com/test/101',
        '_sName': 'Test Profile',
        '_sInitialVisibility': 'public',
        '_bHasFiles': True,
        '_nSubscriberCount': 42,
        '_bShowRipePromo': False
    }

    profile = Profile.from_dict(test_data)

    assert profile.id == 101
    assert profile.status == 1
    assert profile.is_private is False
    assert isinstance(profile.date_modified, datetime)
    assert isinstance(profile.date_added, datetime)
    assert profile.profile_url == 'https://gamebanana.com/test/101'
    assert profile.name == 'Test Profile'
    assert profile.initial_visibility == 'public'
    assert profile.has_files is True
    assert profile.subscriber_count == 42
    assert profile.show_ripe_promo is False

def test_member_profile():
    test_data = {
        # Base profile data
        '_idRow': 102,
        '_sProfileUrl': 'https://gamebanana.com/members/102',
        '_nStatus': 1,
        # Member specific data
        '_sUserTitle': 'Pro Member',
        '_tsJoinDate': 1234567890,
        '_sAvatarUrl': 'https://gamebanana.com/avatars/102.jpg',
        '_sPointsUrl': 'https://gamebanana.com/points/102',
        '_sMedalsUrl': 'https://gamebanana.com/medals/102',
        '_bIsOnline': True,
        '_sOnlineTitle': 'Online Now',
        '_sOfflineTitle': 'Offline',
        '_nPoints': 1000,
        '_nPointsRank': 5,
        '_aBio': [],
        '_aOnlineStatus': {
            '_bIsOnline': True,
            '_sLocation': 'Website',
            '_tsLastSeenTime': 1234567890,
            '_tsSessionCreationTime': 1234567880,
            '_sTitle': 'Online',
            '_sActivity': 'Browsing'
        },
        '_aCoreStats': {
            '_sAccountAge': '2 years',
            '_nCurrentSubmissions': 15,
            '_nCurrentSubscribers': 100,
            '_nThanksReceived': 250,
            '_nPoints': 1000,
            '_nSubmissionsFeatured': 3,
            '_nMedalsCount': 5,
            '_nLikeCount': 50,
            '_nViewCount': 1000,
            '_nPostCount': 25,
            '_nSubmissionCount': 10
        },
        '_bIsBanned': False,
        '_aClearanceLevels': ['user', 'moderator']
    }

    profile = MemberProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.user_title == 'Pro Member'
    assert isinstance(profile.join_date, datetime)
    assert profile.avatar_url == 'https://gamebanana.com/avatars/102.jpg'
    assert profile.points_url == 'https://gamebanana.com/points/102'
    assert profile.medals_url == 'https://gamebanana.com/medals/102'
    assert profile.is_online is True
    assert profile.online_title == 'Online Now'
    assert profile.offline_title == 'Offline'
    assert profile.points == 1000
    assert profile.points_rank == 5
    assert isinstance(profile.bio, list)
    assert isinstance(profile.online_status, OnlineStatus)
    assert isinstance(profile.core_stats, CoreStats)
    assert profile.is_banned is False
    assert profile.clearance_levels == ['user', 'moderator']
    
    # Test CoreStats details
    assert profile.core_stats.account_age == '2 years'
    assert profile.core_stats.current_submissions == 15
    assert profile.core_stats.current_subscribers == 100
    assert profile.core_stats.thanks_received == 250
    assert profile.core_stats.points == 1000
    assert profile.core_stats.submissions_featured == 3
    assert profile.core_stats.medals_count == 5

def test_mod_profile():
    test_data = {
        # Base profile data - required for Profile.from_dict
        '_idRow': 103,
        '_sProfileUrl': 'https://gamebanana.com/mods/103',
        '_sName': 'Test Mod',
        '_nStatus': 1,
        '_bIsPrivate': False,
        '_tsDateModified': 1234567890,
        '_tsDateAdded': 1234567880,
        '_bHasFiles': True,
        '_nSubscriberCount': 50,
        '_bShowRipePromo': False,
        # Mod specific data
        '_sFeedbackInstructions': 'Leave feedback',
        '_bAccessorIsSubmitter': False,
        '_bIsTrashed': False,
        '_bIsWithheld': False,
        '_nUpdatesCount': 5,
        '_bHasUpdates': True,
        '_nAllTodosCount': 2,
        '_nPostCount': 15,
        '_aAttributes': {'category': ['gameplay']},
        '_aTags': [{'_sName': 'test', '_sValue': 'value'}],
        '_bCreatedBySubmitter': True,
        '_bIsPorted': False,
        '_nThanksCount': 20,
        '_sInitialVisibility': 'public',
        '_sDownloadUrl': 'https://gamebanana.com/dl/103',
        '_nDownloadCount': 1000,
        '_aFiles': [],
        '_sDescription': 'Test description',
        '_nLikeCount': 30,
        '_nViewCount': 500
    }

    profile = ModProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.feedback_instructions == 'Leave feedback'
    assert profile.accessor_is_submitter is False
    assert profile.is_trashed is False
    assert profile.is_withheld is False
    assert profile.name == 'Test Mod'
    assert profile.updates_count == 5
    assert profile.has_updates is True
    assert profile.all_todos_count == 2
    assert profile.post_count == 15
    assert profile.attributes == {'category': ['gameplay']}
    assert len(profile.tags) == 1
    assert profile.created_by_submitter is True
    assert profile.is_ported is False
    assert profile.thanks_count == 20
    assert profile.initial_visibility == 'public'
    assert profile.download_url == 'https://gamebanana.com/dl/103'
    assert profile.download_count == 1000
    assert isinstance(profile.files, list)
    assert profile.subscriber_count == 50
    assert profile.description == 'Test description'
    assert profile.like_count == 30
    assert profile.view_count == 500
    assert profile.credits is None
    assert profile.studio is None

def test_game_section():
    test_data = {
        '_sModelName': 'Mod',
        '_sPluralTitle': 'Mods',
        '_nItemCount': 150,
        '_nCategoryCount': 5,
        '_sUrl': 'https://gamebanana.com/games/123/mods'
    }

    section = GameSection.from_dict(test_data)

    assert section.model_name == 'Mod'
    assert section.plural_title == 'Mods'
    assert section.item_count == 150
    assert section.category_count == 5
    assert section.url == 'https://gamebanana.com/games/123/mods'

def test_game_profile():
    test_data = {
        # Base profile data
        '_idRow': 123,
        '_sProfileUrl': 'https://gamebanana.com/games/123',
        '_sName': 'Test Game',
        '_nStatus': 1,
        '_bIsPrivate': False,
        '_tsDateModified': 1234567890,
        '_tsDateAdded': 1234567880,
        # Game specific data
        '_sHomepage': 'https://example.com/game',
        '_bIsApproved': True,
        '_aSections': [{
            '_sModelName': 'Mod',
            '_sPluralTitle': 'Mods',
            '_nItemCount': 150,
            '_nCategoryCount': 5,
            '_sUrl': 'https://gamebanana.com/games/123/mods'
        }],
        '_aModCategories': [],
        '_aManagers': [],
        '_sAbbreviation': 'TG',
        '_tsReleaseDate': 1234567000,
        '_sWelcomeMessage': 'Welcome to Test Game',
        '_sDescription': 'A test game description'
    }

    profile = GameProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.homepage == 'https://example.com/game'
    assert profile.is_approved is True
    assert len(profile.sections) == 1
    assert isinstance(profile.sections[0], GameSection)
    assert isinstance(profile.mod_categories, list)
    assert isinstance(profile.managers, list)
    assert profile.abbreviation == 'TG'
    assert isinstance(profile.release_date, datetime)
    assert profile.welcome_message == 'Welcome to Test Game'
    assert profile.description == 'A test game description'

def test_app_features():
    test_data = {
        '_sProfileModuleUrl': 'https://gamebanana.com/apps/123/profile',
        '_sNavigatorTabUrl': 'https://gamebanana.com/apps/123/nav',
        '_sMainUrl': 'https://gamebanana.com/apps/123',
        '_sSettingsUrl': 'https://gamebanana.com/apps/123/settings'
    }

    features = AppFeatures.from_dict(test_data)

    assert features.profile_module_url == 'https://gamebanana.com/apps/123/profile'
    assert features.navigator_tab_url == 'https://gamebanana.com/apps/123/nav'
    assert features.main_url == 'https://gamebanana.com/apps/123'
    assert features.settings_url == 'https://gamebanana.com/apps/123/settings'

def test_app_profile():
    test_data = {
        # Base profile data
        '_idRow': 456,
        '_sProfileUrl': 'https://gamebanana.com/apps/456',
        '_sName': 'Test App',
        # App specific data
        '_sDescription': 'A test app',
        '_sIncludeVariableName': 'TEST_APP',
        '_sVersion': '1.0.0',
        '_sState': 'stable',
        '_sType': 'tool',
        '_nUserCount': 1000,
        '_sbIsSafe': 'true',
        '_bAcceptsDonations': True,
        '_bAccessorHasApp': False,
        '_aFeatures': {
            '_sProfileModuleUrl': 'https://gamebanana.com/apps/456/profile',
            '_sNavigatorTabUrl': 'https://gamebanana.com/apps/456/nav',
            '_sMainUrl': 'https://gamebanana.com/apps/456',
            '_sSettingsUrl': 'https://gamebanana.com/apps/456/settings'
        }
    }

    profile = AppProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.description == 'A test app'
    assert profile.include_variable_name == 'TEST_APP'
    assert profile.version == '1.0.0'
    assert profile.state == 'stable'
    assert profile.type == 'tool'
    assert profile.user_count == 1000
    assert profile.is_safe is True
    assert profile.accepts_donations is True
    assert profile.accessor_has_app is False
    assert isinstance(profile.features, AppFeatures)

def test_open_position():
    test_data = {
        '_idSkillRow': 'skill123',
        '_sSkill': 'Programmer',
        '_idGameRow': 'game456',
        '_aGame': {'name': 'Test Game', 'url': 'https://example.com'},
        '_sNotes': 'Looking for Python developer'
    }

    position = OpenPosition.from_dict(test_data)

    assert position.skill_id == 'skill123'
    assert position.skill == 'Programmer'
    assert position.game_id == 'game456'
    assert position.game == {'name': 'Test Game', 'url': 'https://example.com'}
    assert position.notes == 'Looking for Python developer'

def test_studio_profile():
    test_data = {
        # Base profile data
        '_idRow': 789,
        '_sProfileUrl': 'https://gamebanana.com/studios/789',
        '_sName': 'Test Studio',
        # Studio specific data
        '_sMotto': 'Creating awesome stuff',
        '_sJoinMode': 'application',
        '_sFlagUrl': 'https://gamebanana.com/studios/789/flag.png',
        '_sBannerUrl': 'https://gamebanana.com/studios/789/banner.png',
        '_iMemberCount': 10,
        '_nPostCount': 100,
        '_aSocialLinks': [{'platform': 'twitter', 'url': 'https://twitter.com/teststudio'}],
        '_sProfileTemplate': 'default',
        '_aProfileModules': ['info', 'members'],
        '_aOpenPositions': [{
            '_idSkillRow': 'skill123',
            '_sSkill': 'Programmer',
            '_idGameRow': 'game456',
            '_aGame': {'name': 'Test Game', 'url': 'https://example.com'},
            '_sNotes': 'Looking for Python developer'
        }],
        '_aLeadership': [],
        '_bAccessorIsInGuild': False,
        '_bAccessorHasPendingJoinRequest': False
    }

    profile = StudioProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.motto == 'Creating awesome stuff'
    assert profile.join_mode == 'application'
    assert profile.flag_url == 'https://gamebanana.com/studios/789/flag.png'
    assert profile.banner_url == 'https://gamebanana.com/studios/789/banner.png'
    assert profile.member_count == 10
    assert profile.post_count == 100
    assert len(profile.social_links) == 1
    assert profile.profile_template == 'default'
    assert profile.profile_modules == ['info', 'members']
    assert len(profile.open_positions) == 1
    assert isinstance(profile.open_positions[0], OpenPosition)
    assert isinstance(profile.leadership, list)
    assert profile.accessor_is_in_guild is False
    assert profile.accessor_has_pending_join_request is False

def test_license_checklist():
    test_data = {
        'yes': ['modify', 'distribute'],
        'ask': ['commercial'],
        'no': ['sell']
    }

    checklist = LicenseChecklist.from_dict(test_data)

    assert checklist.yes == ['modify', 'distribute']
    assert checklist.ask == ['commercial']
    assert checklist.no == ['sell']

def test_preview_media():
    test_data = {
        '_aImages': [{
            '_sUrl': 'https://images.gamebanana.com/123/preview.jpg',
            '_nWidth': 800,
            '_nHeight': 600,
            '_nFilesize': 12345
        }],
        '_aMetadata': {
            'title': 'Preview Image',
            'description': 'A test preview image'
        }
    }

    preview = PreviewMedia.from_dict(test_data)

    assert len(preview.images) == 1
    assert isinstance(preview.images[0], PreviewMediaImage)
    assert preview.images[0].url == 'https://images.gamebanana.com/123/preview.jpg'
    assert preview.images[0].width == 800
    assert preview.images[0].height == 600
    assert preview.images[0].file_size == 12345
    assert preview.metadata == {'title': 'Preview Image', 'description': 'A test preview image'}

def test_preview_media_empty():
    test_data = {
        '_aImages': [],
        '_aMetadata': {}
    }

    preview = PreviewMedia.from_dict(test_data)

    assert len(preview.images) == 0
    assert preview.metadata == {}

def test_preview_media_none():
    preview = PreviewMedia.from_dict({'_aImages': None, '_aMetadata': None})

    assert len(preview.images) == 0
    assert preview.metadata == {}

def test_moderator_response():
    test_data = {
        '_aRecords': [
            {
                '_aMember': {'_idRow': 1, '_sName': 'Mod1'},
                '_sStaffBio': 'Bio 1',
                '_aModgroups': ['admin']
            },
            {
                '_aMember': {'_idRow': 2, '_sName': 'Mod2'},
                '_sStaffBio': 'Bio 2',
                '_aModgroups': ['moderator']
            }
        ]
    }

    response = ModeratorResponse.from_dict(test_data)
    assert len(response.records) == 2
    assert isinstance(response.records[0], ModeratorRecord)
    assert isinstance(response.records[1], ModeratorRecord)
    assert response.records[0].member.id == 1
    assert response.records[1].member.name == 'Mod2'
    
def test_game_manager_response():
    test_data = {
        '_aMetadata': {'total': 2},
        '_aRecords': [
            {
                '_aMember': {'_idRow': 1, '_sName': 'Manager1'},
                '_aModgroups': ['admin'],
                '_tsDateAdded': 1234567890
            },
            {
                '_aMember': {'_idRow': 2, '_sName': 'Manager2'},
                '_aModgroups': ['moderator'],
                '_tsDateAdded': 1234567891
            }
        ]
    }

    response = GameManagerResponse.from_dict(test_data)
    assert response.metadata == {'total': 2}
    assert len(response.records) == 2
    assert isinstance(response.records[0], ManagerRecord)
    assert isinstance(response.records[1], ManagerRecord)
    assert response.records[0].member.id == 1
    assert response.records[1].member.name == 'Manager2'

def test_result_response():
    test_data = {
        '_aRecords': [
            {
                '_idRow': 1,
                '_sName': 'Result1',
                '_sModelName': 'Mod',
                '_sSingularTitle': 'Mod',
                '_sIconClasses': 'icon mod',
                '_sProfileUrl': 'https://gamebanana.com/mods/1',
                '_tsDateAdded': 1234567890,
                '_tsDateModified': 1234567891,
                '_bHasFiles': True
            },
            {
                '_idRow': 2,
                '_sName': 'Result2',
                '_sModelName': 'Game',
                '_sSingularTitle': 'Game',
                '_sIconClasses': 'icon game',
                '_sProfileUrl': 'https://gamebanana.com/games/2',
                '_tsDateAdded': 1234567892,
                '_tsDateModified': 1234567893,
                '_bHasFiles': False
            }
        ],
        '_nRecordCount': 2
    }

    response = ResultResponse.from_dict(test_data)
    assert len(response.records) == 2
    assert response.record_count == 2
    
    result1 = response.records[0]
    assert isinstance(result1, Result)
    assert result1.id_row == 1
    assert result1.name == 'Result1'
    assert result1.model_name == 'Mod'
    assert result1.singular_title == 'Mod'
    assert result1.icon_classes == 'icon mod'
    assert result1.profile_url == 'https://gamebanana.com/mods/1'
    assert result1.date_added == 1234567890
    assert result1.date_modified == 1234567891
    assert result1.has_files is True

    result2 = response.records[1]
    assert isinstance(result2, Result)
    assert result2.id_row == 2
    assert result2.name == 'Result2'
    assert result2.model_name == 'Game'
    assert result2.singular_title == 'Game'

def test_ratings_summary():
    test_data = {
        '_aRatingsSummary': {
            '_nRatingsCount': 100,
            '_iCumulativeRating': 450,
            '_iCumulativePositivity': 400,
            '_iCumulativeNegativity': 50,
            '_aRatingsBreakdown': {
                'likes': {
                    '_nCount': 80,
                    '_sTitle': 'Likes',
                    '_sVerb': 'Liked',
                    '_sIconUrl': 'like.png',
                    '_sIconClasses': 'like-icon'
                }
            }
        }
    }

    ratings = RatingsSummary.from_dict(test_data)

    assert ratings.ratings_count == 100
    assert ratings.cumulative_rating == 450
    assert ratings.cumulative_positivity == 400
    assert ratings.cumulative_negativity == 50
    assert len(ratings.ratings_breakdown) == 1
    assert 'likes' in ratings.ratings_breakdown
    like_item = ratings.ratings_breakdown['likes']
    assert like_item.count == 80
    assert like_item.title == 'Likes'
    assert like_item.verb == 'Liked'
    assert like_item.icon_url == 'like.png'
    assert like_item.icon_classes == 'like-icon'

def test_bug_profile():
    test_data = {
        # Base profile data
        '_idRow': 789,
        '_sProfileUrl': 'https://gamebanana.com/bugs/789',
        '_sName': 'Test Bug',
        # Bug specific data
        '_nStatus': 2,
        '_bIsPrivate': False,
        '_tsDateModified': 1234567890,
        '_tsDateAdded': 1234567880,
        '_aPreviewMedia': {'_aImages': [], '_aMetadata': {}},
        '_bAccessorIsSubmitter': True,
        '_bIsTrashed': False,
        '_nPostCount': 5,
        '_nThanksCount': 10,
        '_sInitialVisibility': 'public',
        '_bHasFiles': False,
        '_nSubscriberCount': 2,
        '_sText': 'Bug description',
        '_sResolution': 'Fixed',
        '_sResolutionKey': 'fixed',
        '_sPriority': 'High',
        '_sPriorityKey': 'high',
        '_sSourceUrl': 'https://example.com/source',
        '_bShowRipePromo': False,
        '_aEmbeddables': [],
        '_aMember': {
            '_idRow': 123,
            '_sName': 'TestUser',
            '_bIsOnline': True,
            '_bHasRipe': False,
            '_sProfileUrl': 'https://gamebanana.com/members/123',
            '_sAvatarUrl': 'https://gamebanana.com/avatars/123.jpg'
        }
    }

    profile = BugProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.status == 2
    assert profile.is_private is False
    assert isinstance(profile.date_modified, datetime)
    assert isinstance(profile.date_added, datetime)
    assert isinstance(profile.preview_media, PreviewMedia)
    assert profile.accessor_is_submitter is True
    assert profile.is_trashed is False
    assert profile.post_count == 5
    assert profile.thanks_count == 10
    assert profile.initial_visibility == 'public'
    assert profile.has_files is False
    assert profile.subscriber_count == 2
    assert profile.text == 'Bug description'
    assert profile.resolution == 'Fixed'
    assert profile.resolution_key == 'fixed'
    assert profile.priority == 'High'
    assert profile.priority_key == 'high'
    assert profile.source_url == 'https://example.com/source'
    assert profile.show_ripe_promo is False
    assert isinstance(profile.embeddables, list)
    assert isinstance(profile.submitter, Member)

def test_idea_profile():
    test_data = {
        # Base profile data
        '_idRow': 456,
        '_sProfileUrl': 'https://gamebanana.com/ideas/456',
        '_sName': 'Test Idea',
        # Idea specific data
        '_sText': 'Idea description',
        '_nPostCount': 3,
        '_bHasRevisions': True,
        '_bHasChangelog': True,
        '_bIsPrivate': False,
        '_bIsShared': True,
        '_nSortingPriority': 1,
        '_bSupportsDownvoting': True,
        '_aRatingsSummary': {
            '_nRatingsCount': 50,
            '_iCumulativeRating': 42,
            '_iCumulativePositivity': 40,
            '_iCumulativeNegativity': 2,
            '_aRatingsBreakdown': {}
        },
        '_aEmbeddables': []
    }

    profile = IdeaProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.text == 'Idea description'
    assert profile.post_count == 3
    assert profile.has_revisions is True
    assert profile.has_changelog is True
    assert profile.is_private is False
    assert profile.is_shared is True
    assert profile.sorting_priority == 1
    assert profile.supports_downvoting is True
    assert profile.ratings_summary is not None
    assert profile.ratings_summary.ratings_count == 50
    assert profile.ratings_summary.cumulative_rating == 42
    assert profile.ratings_summary.cumulative_positivity == 40
    assert profile.ratings_summary.cumulative_negativity == 2
    assert isinstance(profile.ratings_summary.ratings_breakdown, dict)
    assert isinstance(profile.embeddables, list)

def test_bio():
    test_data = {
        '_aBio': [{
            '_sTitle': 'About Me',
            '_sValue': 'I make games',
            '_sCustomTitle': 'Personal Info'
        }]
    }

    bio = Bio.from_dict(test_data)

    assert len(bio.entries) == 1
    entry = bio.entries[0]
    assert entry.title == 'About Me'
    assert entry.value == 'I make games'
    assert entry.custom_title == 'Personal Info'

def test_author():
    test_data = {
        '_sRole': 'Lead Developer',
        '_idRow': 123,
        '_sName': 'TestAuthor',
        '_sUpicUrl': 'https://example.com/pic.jpg',
        '_sProfileUrl': 'https://gamebanana.com/members/123',
        '_bIsOnline': True
    }

    author = Author.from_dict(test_data)

    assert author.role == 'Lead Developer'
    assert author.id == 123
    assert author.name == 'TestAuthor'
    assert author.upic_url == 'https://example.com/pic.jpg'
    assert author.profile_url == 'https://gamebanana.com/members/123'
    assert author.is_online is True

def test_credit_group():
    test_data = {
        '_sGroupName': 'Development Team',
        '_aAuthors': [
            {
                '_sRole': 'Lead Developer',
                '_idRow': 123,
                '_sName': 'TestAuthor',
                '_sUpicUrl': 'https://example.com/pic.jpg',
                '_sProfileUrl': 'https://gamebanana.com/members/123',
                '_bIsOnline': True
            }
        ]
    }

    group = CreditGroup.from_dict(test_data)

    assert group.group_name == 'Development Team'
    assert len(group.authors) == 1
    assert isinstance(group.authors[0], Author)
    assert group.authors[0].role == 'Lead Developer'

def test_mod_category():
    test_data = {
        '_idRow': 42,
        '_sName': 'Gameplay Mods',
        '_nItemCount': 100,
        '_nCategoryCount': 5,
        '_sUrl': 'https://gamebanana.com/games/123/mods/gameplay'
    }

    category = ModCategory.from_dict(test_data)

    assert category.id == 42
    assert category.name == 'Gameplay Mods'
    assert category.item_count == 100
    assert category.category_count == 5
    assert category.url == 'https://gamebanana.com/games/123/mods/gameplay'

def test_embeddable():
    test_data = {
        '_sEmbeddableImageBaseUrl': 'https://images.gamebanana.com/embed/123',
        '_aVariants': ['small', 'medium', 'large']
    }

    embeddable = Embeddable.from_dict(test_data)

    assert embeddable.image_base_url == 'https://images.gamebanana.com/embed/123'
    assert embeddable.variants == ['small', 'medium', 'large']

def test_embeddable_string_input():
    # For the string input case, we need to wrap it in a dict to match the type signature
    test_data = {'_sEmbeddableImageBaseUrl': 'https://images.gamebanana.com/embed/123'}
    embeddable = Embeddable.from_dict(test_data)
    assert embeddable.image_base_url == 'https://images.gamebanana.com/embed/123'
    assert embeddable.variants == []

def test_file():
    test_data = {
        '_idRow': 789,
        '_sFile': 'test_mod.zip',
        '_nFilesize': 1024000,
        '_sDescription': 'Main mod file',
        '_tsDateAdded': 1234567890,
        '_nDownloadCount': 5000,
        '_sAnalysisState': 'completed',
        '_sAnalysisResultCode': 'clean',
        '_sAnalysisResult': 'No issues found',
        '_bContainsExe': False,
        '_sDownloadUrl': 'https://gamebanana.com/dl/789',
        '_sMd5Checksum': 'abc123def456',
        '_sClamAvResult': 'clean',
        '_sAvastAvResult': 'clean'
    }

    file = File.from_dict(test_data)

    assert file.id == 789
    assert file.filename == 'test_mod.zip'
    assert file.filesize == 1024000
    assert file.description == 'Main mod file'
    assert isinstance(file.date_added, datetime)
    assert file.download_count == 5000
    assert file.analysis_state == 'completed'
    assert file.analysis_result_code == 'clean'
    assert file.analysis_result == 'No issues found'
    assert file.contains_exe is False
    assert file.download_url == 'https://gamebanana.com/dl/789'
    assert file.md5_checksum == 'abc123def456'
    assert file.clam_av_result == 'clean'
    assert file.avast_av_result == 'clean'

def test_club_profile():
    test_data = {
        # Base profile data
        '_idRow': 234,
        '_sProfileUrl': 'https://gamebanana.com/clubs/234',
        '_sName': 'Test Club',
        # Club specific data
        '_nStatus': 1,
        '_bIsPrivate': False,
        '_tsDateModified': 1234567890,
        '_tsDateAdded': 1234567880,
        '_aPreviewMedia': {'_aImages': [], '_aMetadata': {}},
        '_bAccessorIsSubmitter': True,
        '_bIsTrashed': False,
        '_nPostCount': 10,
        '_sInitialVisibility': 'public',
        '_bHasFiles': False,
        '_sText': 'Club description',
        '_nMemberCount': 25,
        '_tsLastActivityDate': 1234567890,
        '_bShowRipePromo': False,
        '_aMember': {
            '_idRow': 123,
            '_sName': 'TestUser',
            '_bIsOnline': True,
            '_bHasRipe': False,
            '_sProfileUrl': 'https://gamebanana.com/members/123',
            '_sAvatarUrl': 'https://gamebanana.com/avatars/123.jpg'
        },
        '_sFlagUrl': 'https://gamebanana.com/clubs/234/flag.png',
        '_sBannerUrl': 'https://gamebanana.com/clubs/234/banner.png',
        '_sMotto': 'Club motto',
        '_sJoinMode': 'open',
        '_aSocialLinks': ['https://twitter.com/testclub'],
        '_sProfileTemplate': 'default',
        '_aProfileModules': ['info', 'members'],
        '_bAccessorIsInGuild': False,
        '_bAccessorHasPendingJoinRequest': False,
        '_aLeadership': [
            {
                '_aMember': {
                    '_idRow': 456,
                    '_sName': 'LeaderUser',
                    '_bIsOnline': True,
                    '_bHasRipe': True,
                    '_sProfileUrl': 'https://gamebanana.com/members/456',
                    '_sAvatarUrl': 'https://gamebanana.com/avatars/456.jpg'
                },
                '_aModgroups': ['admin', 'moderator'],
                '_tsDateAdded': 1234567000
            }
        ]
    }

    profile = ClubProfile.from_dict(test_data)

    assert isinstance(profile.base, Profile)
    assert profile.status == 1
    assert profile.is_private is False
    assert isinstance(profile.date_modified, datetime)
    assert isinstance(profile.date_added, datetime)
    assert isinstance(profile.preview_media, PreviewMedia)
    assert profile.accessor_is_submitter is True
    assert profile.is_trashed is False
    assert profile.name == 'Test Club'
    assert profile.post_count == 10
    assert profile.initial_visibility == 'public'
    assert profile.has_files is False
    assert profile.text == 'Club description'
    assert profile.member_count == 25
    assert isinstance(profile.last_activity_date, datetime)
    assert profile.show_ripe_promo is False
    assert isinstance(profile.submitter, Member)
    assert profile.flag_url == 'https://gamebanana.com/clubs/234/flag.png'
    assert profile.banner_url == 'https://gamebanana.com/clubs/234/banner.png'
    assert profile.motto == 'Club motto'
    assert profile.join_mode == 'open'
    assert profile.social_links == ['https://twitter.com/testclub']
    assert profile.profile_template == 'default'
    assert profile.profile_modules == ['info', 'members']
    assert profile.accessor_is_in_guild is False
    assert profile.accessor_has_pending_join_request is False
    
    # Test leadership records
    assert len(profile.leadership) == 1
    leader = profile.leadership[0]
    assert isinstance(leader, ManagerRecord)
    assert isinstance(leader.member, Member)
    assert leader.member.id == 456
    assert leader.member.name == 'LeaderUser'
    assert leader.member.is_online is True
    assert leader.member.has_ripe is True
    assert leader.modgroups == ['admin', 'moderator']
    assert isinstance(leader.date_added, int)
    assert leader.date_added == 1234567000

def test_affiliated_studio():
    test_data = {
        '_sProfileUrl': 'https://gamebanana.com/studios/789',
        '_sName': 'Partner Studio',
        '_sFlagUrl': 'https://gamebanana.com/studios/789/flag.png',
        '_sBannerUrl': 'https://gamebanana.com/studios/789/banner.png'
    }

    studio = AffiliatedStudio.from_dict(test_data)

    assert studio.profile_url == 'https://gamebanana.com/studios/789'
    assert studio.name == 'Partner Studio'
    assert studio.flag_url == 'https://gamebanana.com/studios/789/flag.png'
    assert studio.banner_url == 'https://gamebanana.com/studios/789/banner.png'

def test_subject_shaper():
    test_data = {
        "_sBorderStyle": "solid",
        "_sFont": "Arial",
        "_sTextColor": "#000000",
        "_sTextHoverColor": "#555555",
        "_sBorderColor": "#cccccc",
        "_sBorderHoverColor": "#999999"
    }

    shaper = SubjectShaper.from_dict(test_data)

    assert shaper.border_style == "solid"
    assert shaper.font == "Arial"
    assert shaper.text_color == "#000000"
    assert shaper.text_hover_color == "#555555"
    assert shaper.border_color == "#cccccc"
    assert shaper.border_hover_color == "#999999"

def test_subject_shaper_empty():
    test_data = {}
    
    shaper = SubjectShaper.from_dict(test_data)

    assert shaper.border_style == ""
    assert shaper.font == ""
    assert shaper.text_color == ""
    assert shaper.text_hover_color == ""
    assert shaper.border_color == ""
    assert shaper.border_hover_color == ""

def test_buddy():
    test_data = {
        "_aBuddy": {
            "_idRow": 123,
            "_sName": "TestBuddy",
            "_bIsOnline": True,
            "_bHasRipe": False,
            "_sProfileUrl": "https://gamebanana.com/members/123",
            "_sAvatarUrl": "https://gamebanana.com/avatars/123.jpg",
            "_aClearanceLevels": ["user", "moderator"],
            "_sHdAvatarUrl": "https://gamebanana.com/avatars/hd/123.jpg",
            "_sUpicUrl": "https://gamebanana.com/upics/123.jpg",
            "_aSubjectShaper": {
                "_sBorderStyle": "solid",
                "_sFont": "Arial",
                "_sTextColor": "#000000",
                "_sTextHoverColor": "#555555",
                "_sBorderColor": "#cccccc",
                "_sBorderHoverColor": "#999999"
            },
            "_sSubjectShaperCssCode": ".buddy-123 { border: solid 1px #cccccc; }"
        },
        "_tsDateAdded": 1234567890
    }

    buddy = Buddy.from_dict(test_data)

    # Test inherited Member fields
    assert buddy.id == 123
    assert buddy.name == "TestBuddy"
    assert buddy.is_online is True
    assert buddy.has_ripe is False
    assert buddy.profile_url == "https://gamebanana.com/members/123"
    assert buddy.avatar_url == "https://gamebanana.com/avatars/123.jpg"

    # Test Buddy-specific fields
    assert buddy.clearance_levels == ["user", "moderator"]
    assert buddy.hd_avatar_url == "https://gamebanana.com/avatars/hd/123.jpg"
    assert buddy.upic_url == "https://gamebanana.com/upics/123.jpg"
    assert isinstance(buddy.subject_shaper, SubjectShaper)
    assert buddy.subject_shaper.border_style == "solid"
    assert buddy.subject_shaper_css_code == ".buddy-123 { border: solid 1px #cccccc; }"
    assert isinstance(buddy.date_added, datetime)
    assert buddy.date_added.timestamp() == 1234567890

def test_buddy_empty():
    test_data = {
        "_aBuddy": {
            "_idRow": 123,
            "_sName": "TestBuddy",
            "_bIsOnline": False,
            "_bHasRipe": False,
            "_sProfileUrl": "https://gamebanana.com/members/123",
            "_sAvatarUrl": "https://gamebanana.com/avatars/123.jpg"
        },
        "_tsDateAdded": 0
    }

    buddy = Buddy.from_dict(test_data)

    assert buddy.id == 123
    assert buddy.name == "TestBuddy"
    assert buddy.clearance_levels == []
    assert buddy.hd_avatar_url == ""
    assert buddy.upic_url == ""
    assert isinstance(buddy.subject_shaper, SubjectShaper)
    assert buddy.subject_shaper.border_style == ""
    assert buddy.subject_shaper_css_code == ""
    assert buddy.date_added is None


"""Profile models for different GameBanana content types."""
from .app import AppProfile
from .game import GameProfile
from .member import MemberProfile
from .idea import IdeaProfile
from .bug import BugProfile
from .mod import ModProfile
from .studio import StudioProfile
from .club import ClubProfile

__all__ = [
    'AppProfile',
    'GameProfile',
    'MemberProfile',
    'IdeaProfile',
    'BugProfile',
    'ModProfile',
    'StudioProfile',
    'ClubProfile',
]